@@ -1,10 +1,13 @@
 /* ============================================================
- * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- * Date  : 2004-06-15
- * Description : 
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
  * 
- * Copyright 2004 by Renchi Raju
-
+ * Date        : 2004-06-15
+ * Description : Albums manager interface.
+ * 
+ * Copyright (C) 2004 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
+ *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
@@ -18,8 +21,37 @@
  * 
  * ============================================================ */
 
+#include <config.h>
+
+// C Ansi includes.
+
+extern "C"
+{
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <locale.h>
+}
+
+// C++ includes.
+
+#include <cstdlib>
+#include <cstdio>
+#include <cerrno> 
+
+
+// Qt includes.
+
+#include <qfile.h>
+#include <qdir.h>
+#include <qdict.h>
+#include <qintdict.h>
+#include <qcstring.h>
+#include <qtextcodec.h>
+
+// KDE includes.
+
 #include <kconfig.h>
-#include <kdebug.h>
 #include <klocale.h>
 #include <kdeversion.h>
 #include <kmessagebox.h>
@@ -29,15 +61,9 @@
 #include <kio/job.h>
 #include <kdirwatch.h>
 
-#include <qfile.h>
-#include <qdir.h>
-#include <qdict.h>
-#include <qintdict.h>
-#include <qcstring.h>
-#include <qtextcodec.h>
-
-#include <config.h>
+// Local includes.
 
+#include "ddebug.h"
 #include "album.h"
 #include "albumdb.h"
 #include "albumitemhandler.h"
@@ -46,17 +72,10 @@
 #include "scanlib.h"
 #include "upgradedb_sqlite2tosqlite3.h"
 #include "albummanager.h"
+#include "albummanager.moc"
 
-extern "C"
+namespace Digikam
 {
-#include <errno.h> 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <locale.h>
-}
 
 typedef QDict<PAlbum>    PAlbumDict;
 typedef QIntDict<Album>  AlbumIntDict;
@@ -200,7 +219,7 @@ void AlbumManager::setLibraryPath(const QString& path)
     
     if (dbLocale.isNull())
     {
-        kdDebug() << "No locale found in database" << endl;
+        DDebug() << "No locale found in database" << endl;
 
         // Copy an existing locale from the settings file (used < 0.8)
         // to the database.
@@ -208,7 +227,7 @@ void AlbumManager::setLibraryPath(const QString& path)
         config->setGroup("General Settings");
         if (config->hasKey("Locale"))
         {
-            kdDebug() << "Locale found in configfile" << endl;
+            DDebug() << "Locale found in configfile" << endl;
             dbLocale = config->readEntry("Locale");
 
             // this hack is necessary, as we used to store the entire
@@ -226,7 +245,7 @@ void AlbumManager::setLibraryPath(const QString& path)
         }
         else
         {
-            kdDebug() << "No locale found in config file"  << endl;
+            DDebug() << "No locale found in config file"  << endl;
             dbLocale = currLocale;
 
             localeChanged = false;
@@ -241,6 +260,8 @@ void AlbumManager::setLibraryPath(const QString& path)
 
     if (localeChanged)
     {
+        // TODO it would be better to replace all yes/no confirmation dialogs with ones that has custom
+        // buttons that denote the actions directly, i.e.:  ["Ignore and Continue"]  ["Adjust locale"]
         int result =
             KMessageBox::warningYesNo(0,
                                       i18n("Your locale has changed from the previous time "
@@ -248,8 +269,8 @@ void AlbumManager::setLibraryPath(const QString& path)
                                            "Old Locale : %1, New Locale : %2\n"
                                            "This can cause unexpected problems. "
                                            "If you are sure that you want to "
-                                           "continue, click on 'Yes' to work with this album. "
-                                           "Otherwise, click on 'No' and correct your "
+                                           "continue, click 'Yes' to work with this album. "
+                                           "Otherwise, click 'No' and correct your "
                                            "locale setting before restarting digiKam")
                                       .arg(dbLocale)
                                       .arg(currLocale));
@@ -406,7 +427,7 @@ void AlbumManager::scanPAlbums()
         PAlbum* parent = d->pAlbumDict.find(purl);
         if (!parent)
         {
-            kdWarning() << k_funcinfo <<  "Could not find parent with url: "
+            DWarning() << k_funcinfo <<  "Could not find parent with url: "
                         << purl << " for: " << info.url << endl;
             continue;
         }
@@ -483,7 +504,7 @@ void AlbumManager::scanTAlbums()
             }
             else
             {
-                kdWarning() << "Failed to find parent tag for tag "
+                DWarning() << "Failed to find parent tag for tag "
                             << iter.current()->m_title
                             << " with pid "
                             << iter.current()->m_pid << endl;
@@ -520,7 +541,7 @@ void AlbumManager::scanTAlbums()
         TagMap::iterator iter = tmap.find(info.pid);
         if (iter == tmap.end())
         {
-            kdWarning() << "Failed to find parent tag for tag "
+            DWarning() << "Failed to find parent tag for tag "
                         << info.name 
                         << " with pid "
                         << info.pid << endl;
@@ -595,16 +616,19 @@ void AlbumManager::scanDAlbums()
     ds << d->libraryPath;
     ds << KURL();
     ds << AlbumSettings::instance()->getAllFileFilter();
-
+    ds << 0; // getting dimensions (not needed here)
+    ds << 0; // recursive sub-album (not needed here)
+    ds << 0; // recursive sub-tags (not needed here)
     
     d->dateListJob = new KIO::TransferJob(u, KIO::CMD_SPECIAL,
                                           ba, QByteArray(), false);
     d->dateListJob->addMetaData("folders", "yes");
 
     connect(d->dateListJob, SIGNAL(result(KIO::Job*)),
-            SLOT(slotResult(KIO::Job*)));
+            this, SLOT(slotResult(KIO::Job*)));
+
     connect(d->dateListJob, SIGNAL(data(KIO::Job*, const QByteArray&)),
-            SLOT(slotData(KIO::Job*, const QByteArray&)));
+            this, SLOT(slotData(KIO::Job*, const QByteArray&)));
 }
 
 AlbumList AlbumManager::allPAlbums() const
@@ -736,6 +760,22 @@ Album* AlbumManager::findAlbum(int gid) const
     return d->albumIntDict.find(gid);
 }
 
+TAlbum* AlbumManager::findTAlbum(const QString &tagPath) const
+{
+    // handle gracefully with or without leading slash
+    bool withLeadingSlash = tagPath.startsWith("/");
+    AlbumIterator it(d->rootTAlbum);
+    while (it.current())
+    {
+        TAlbum *talbum = static_cast<TAlbum *>(*it);
+        if (talbum->tagPath(withLeadingSlash) == tagPath)
+            return talbum;
+        ++it;
+    }
+    return 0;
+
+}
+
 
 PAlbum* AlbumManager::createPAlbum(PAlbum* parent,
                                    const QString& name,
@@ -769,14 +809,14 @@ PAlbum* AlbumManager::createPAlbum(PAlbum* parent,
     {
         if (child->title() == name)
         {
-            errMsg = i18n("Another album with same name exists.");
+            errMsg = i18n("An existing album has the same name.");
             return 0;
         }
         child = child->m_next;
     }
 
     QString path = parent->folderPath();
-    path += "/" + name;
+    path += '/' + name;
     path = QDir::cleanDirPath(path);
 
     // make the directory synchronously, so that we can add the
@@ -788,7 +828,7 @@ PAlbum* AlbumManager::createPAlbum(PAlbum* parent,
         else if (errno == EACCES)
             errMsg = i18n("Access denied to path");
         else if (errno == ENOSPC)
-            errMsg = i18n("Disk full");
+            errMsg = i18n("Disk is full");
         else
             errMsg = i18n("Unknown error"); // being lazy
 
@@ -848,7 +888,7 @@ bool AlbumManager::renamePAlbum(PAlbum* album, const QString& newName,
     {
         if (sibling->title() == newName)
         {
-            errMsg = i18n("Another Album with same name exists\n"
+            errMsg = i18n("Another album with same name exists\n"
                           "Please choose another name");
             return false;
         }
@@ -953,7 +993,7 @@ TAlbum* AlbumManager::createTAlbum(TAlbum* parent, const QString& name,
     {
         if (child->title() == name)
         {
-            errMsg = i18n("Another tag with same name exists");
+            errMsg = i18n("Tag name already exists");
             return 0;
         }
         child = child->m_next;
@@ -975,6 +1015,26 @@ TAlbum* AlbumManager::createTAlbum(TAlbum* parent, const QString& name,
     return album;
 }
 
+AlbumList AlbumManager::findOrCreateTAlbums(const QStringList &tagPaths)
+{
+    IntList tagIDs;
+
+    // find tag ids for tag paths in list, create if they don't exist
+    tagIDs = d->db->getTagsFromTagPaths(tagPaths);
+
+    // create TAlbum objects for the newly created tags
+    scanTAlbums();
+
+    AlbumList resultList;
+
+    for (IntList::iterator it = tagIDs.begin(); it != tagIDs.end(); ++it)
+    {
+        resultList.append(findTAlbum(*it));
+    }
+
+    return resultList;
+}
+
 bool AlbumManager::deleteTAlbum(TAlbum* album, QString& errMsg)
 {
     if (!album)
@@ -1261,7 +1321,7 @@ void AlbumManager::slotResult(KIO::Job* job)
 
     if (job->error())
     {
-        kdWarning() << k_funcinfo << "Failed to list dates" << endl;
+        DWarning() << k_funcinfo << "Failed to list dates" << endl;
         return;
     }
     
@@ -1329,7 +1389,7 @@ void AlbumManager::slotDirty(const QString& path)
     if (d->dirtyAlbums.contains(url))
         return;
 
-    kdDebug() << "Dirty: " << url << endl;
+    DDebug() << "Dirty: " << url << endl;
     d->dirtyAlbums.append(url);
 
     if (DIO::running())
@@ -1343,5 +1403,6 @@ void AlbumManager::slotDirty(const QString& path)
     DIO::scan(u);
 }
 
+}  // namespace Digikam
+
 
-#include "albummanager.moc"
