@@ -1,11 +1,13 @@
 /* ============================================================
- * Authors: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- *          Caulier Gilles 
- * Date   : 2005-04-24
- * Description : 
  *
- * Copyright 2005 by Renchi Raju
- * Copyright 2006-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ *
+ * Date        : 2005-04-24
+ * Description : icons view.
+ *
+ * Copyright (C) 2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
+ * Copyright (C) 2006-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -22,12 +24,9 @@
 
 #define RECT_EXTENSION 300
 
-// C Ansi includes.
+// C++ includes.
 
-extern "C"
-{
-#include <stdlib.h>
-}
+#include <cstdlib>
 
 // Qt includes.
 
@@ -228,6 +227,29 @@ IconItem* IconView::findItem(const QPoint& pos)
     return 0;
 }
 
+IconGroupItem* IconView::findGroup(const QPoint& pos)
+{
+    QPoint p = viewportToContents(viewport()->mapFromGlobal(pos));
+    for (IconGroupItem* group = d->firstGroup; group; group = group->nextGroup())
+    {
+        QRect rect = group->rect();
+        int bottom;
+        if (group == d->lastGroup)
+            bottom = contentsHeight();
+        else
+            bottom = group->nextGroup()->rect().top();
+
+        rect.setBottom(bottom);
+
+        if ( rect.contains(p) ) 
+        {
+            return group;
+        }
+    }
+
+    return 0;
+}
+
 int IconView::count() const
 {
     int c = 0;
@@ -385,6 +407,11 @@ void IconView::selectItem(IconItem* item, bool select)
     emit signalSelectionChanged();
 }
 
+void IconView::setStoredVisibleItem(IconItem *item) 
+{ 
+    d->storedVisibleItem = item; 
+}
+
 void IconView::insertGroup(IconGroupItem* group)
 {
     if (!group)
@@ -1187,6 +1214,9 @@ void IconView::contentsMouseMoveEvent(QMouseEvent* e)
     p.end();
 
     d->pressedMoved = true;
+
+    if (changed)
+        emit signalSelectionChanged();
 }
 
 void IconView::contentsMouseReleaseEvent(QMouseEvent* e)
