@@ -1,7 +1,12 @@
 /* ============================================================
- * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- * Date  : 2005-06-05
- * Copyright 2005 by Renchi Raju
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ * 
+ * Date        : 2005-06-05
+ * Description : SQlite 2 to SQlite 3 interface
+ *
+ * Copyright (C) 2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -13,6 +18,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
+ *
  * ============================================================ */
 
 #ifndef UPGRADEDB_SQLITE2TOSQLITE3_H
@@ -22,7 +28,11 @@
 
 class QString;
 
+namespace Digikam
+{
+
 extern bool DIGIKAM_EXPORT upgradeDB_Sqlite2ToSqlite3(const QString& libraryPath);
 
+}  // namespace Digikam
 
 #endif /* UPGRADEDB_SQLITE2TOSQLITE3_H */
