@@ -1,10 +1,13 @@
 /* ============================================================
- * File  : digikamalbums.cpp
- * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- * Date  : 2005-04-21
- * Description : 
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ *
+ * Date        : 2005-04-21
+ * Description : a kio-slave to process file operations on 
+ *               digiKam albums. 
  * 
- * Copyright 2005 by Renchi Raju
+ * Copyright (C) 2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
  *
  * Lots of the file io code is copied from KDE file kioslave.
  * Copyright for the KDE file kioslave follows:
@@ -25,7 +28,36 @@
  * 
  * ============================================================ */
 
-#include <digikam_export.h>
+#define MAX_IPC_SIZE (1024*32)
+
+// C Ansi includes.
+
+extern "C" 
+{
+#include <unistd.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/time.h>
+#include <utime.h>
+}
+
+// C++ includes.
+
+#include <cstdlib>
+#include <cstdio>
+#include <ctime>
+#include <cerrno>
+
+// Qt includes.
+
+#include <qfile.h>
+#include <qfileinfo.h>
+#include <qdatastream.h>
+#include <qregexp.h>
+#include <qdir.h>
+
+// KDE includes.
 
 #include <kglobal.h>
 #include <klocale.h>
@@ -36,33 +68,19 @@
 #include <kio/global.h>
 #include <kio/ioslave_defaults.h>
 #include <klargefile.h>
+#include <kdeversion.h>
 
-#include <qfile.h>
-#include <qdatastream.h>
-#include <qregexp.h>
-#include <qdir.h>
+// LibKDcraw includes.
 
-extern "C" 
-{
-#include <stdlib.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#include <time.h>
-#include <utime.h>
-}
+#include <libkdcraw/rawfiles.h>
 
-#include <jpegmetadata.h>
+// Local includes.
 
+#include "dmetadata.h"
 #include "sqlitedb.h"
+#include "digikam_export.h"
 #include "digikamalbums.h"
 
-#define MAX_IPC_SIZE (1024*32)
-
 kio_digikamalbums::kio_digikamalbums(const QCString &pool_socket,
                                      const QCString &app_socket)
     : SlaveBase("kio_digikamalbums", pool_socket, app_socket)
@@ -110,15 +128,20 @@ void kio_digikamalbums::special(const QByteArray& data)
     QString libraryPath;
     KURL    kurl;
     QString url;
+    QString urlWithTrailingSlash;
     QString filter;
     int     getDimensions;
     int     scan = 0;
-    
+    int     recurseAlbums;
+    int     recurseTags;
+
     QDataStream ds(data, IO_ReadOnly);
     ds >> libraryPath;
     ds >> kurl;
     ds >> filter;
     ds >> getDimensions;
+    ds >> recurseAlbums;
+    ds >> recurseTags;
     if (!ds.atEnd())
         ds >> scan;
 
@@ -131,7 +154,7 @@ void kio_digikamalbums::special(const QByteArray& data)
         m_sqlDB.openDB(libraryPath);
     }
 
-    url = QDir::cleanDirPath(kurl.path());
+    url = kurl.path();
     
     if (scan)
     {
@@ -142,72 +165,132 @@ void kio_digikamalbums::special(const QByteArray& data)
     
     QValueList<QRegExp> regex = makeFilterList(filter);
     
-    QStringList values;
-    m_sqlDB.execSql(QString("SELECT id FROM Albums WHERE url='%1';")
-                    .arg(escapeString(url)), &values);
-    int albumid = values.first().toInt();
+    QStringList albumvalues;
+    if (recurseAlbums)
+    {
+        // Search for albums and sub-albums:
+        // For this, get the path with a trailing "/".
+        // Otherwise albums on the same level like "Paris", "Paris 2006",
+        // would be found in addition to "Paris/*".
+        urlWithTrailingSlash = kurl.path(1);
 
-    
-    values.clear();
-    m_sqlDB.execSql(QString("SELECT id, name, datetime FROM Images "
-                                    "WHERE dirid = %1;")
-                            .arg(albumid), &values);
+        m_sqlDB.execSql(QString("SELECT DISTINCT id, url FROM Albums WHERE  url='%1' OR url LIKE '\%%2\%';")
+                        .arg(escapeString(url)).arg(escapeString(urlWithTrailingSlash)), &albumvalues);
+    }
+    else
+    {
+        // Search for albums 
+
+        m_sqlDB.execSql(QString("SELECT DISTINCT id, url FROM Albums WHERE url='%1';")
+                        .arg(escapeString(url)), &albumvalues);
+    }
 
     QByteArray  ba;
-    QDataStream os(ba, IO_WriteOnly);
-    
-    QString base = libraryPath + url + "/";
+    QDataStream* os = new QDataStream(ba, IO_WriteOnly);
+
+    QString base;  
     Q_LLONG id;
     QString name;
     QString date;
     QSize   dims;
 
     struct stat stbuf;
-    for (QStringList::iterator it = values.begin(); it != values.end();)
+    
+    QStringList values;
+    QString albumurl;
+    int albumid;
+
+    // Loop over all albums:
+    int count = 0 ;
+    for (QStringList::iterator albumit = albumvalues.begin(); albumit != albumvalues.end();)
     {
-        id   = (*it).toLongLong();
-        ++it;
-        name = *it;
-        ++it;
-        date = *it;
-        ++it;
+        albumid = (*albumit).toLongLong();
+        ++albumit;
+        albumurl = *albumit;
+        ++albumit;
 
-        if (!matchFilterList(regex, name))
-            continue;
+        base = libraryPath + albumurl + '/';
 
-        if (::stat(QFile::encodeName(base + name), &stbuf) != 0)
-            continue;
+        values.clear();
+        m_sqlDB.execSql(QString("SELECT id, name, datetime FROM Images "
+                                "WHERE dirid = %1;")
+                        .arg(albumid), &values);
 
-        dims = QSize();
-        if (getDimensions)
+        // Loop over all images in each album (specified by its albumid).
+        for (QStringList::iterator it = values.begin(); it != values.end();)
         {
-            KFileMetaInfo metaInfo(base + name);
-            if (metaInfo.isValid())
+            id   = (*it).toLongLong();
+            ++it;
+            name = *it;
+            ++it;
+            date = *it;
+            ++it;
+
+            if (!matchFilterList(regex, name))
+              continue;
+
+            if (::stat(QFile::encodeName(base + name), &stbuf) != 0)
+              continue;
+
+            dims = QSize();
+            if (getDimensions)
             {
-                if (metaInfo.containsGroup("Jpeg EXIF Data"))
+                QFileInfo fileInfo(base + name);
+                QString rawFilesExt(raw_file_extentions);
+                QString ext = fileInfo.extension(false).upper();
+
+                if (!ext.isEmpty() && rawFilesExt.upper().contains(ext))
                 {
-                    dims = metaInfo.group("Jpeg EXIF Data").
-                           item("Dimensions").value().toSize();
+                    Digikam::DMetadata metaData(base + name);
+                    dims = metaData.getImageDimensions();
                 }
-                else if (metaInfo.containsGroup("General"))
+                else
                 {
-                    dims = metaInfo.group("General").
-                           item("Dimensions").value().toSize();
-                }
-                else if (metaInfo.containsGroup("Technical"))
-                {
-                    dims = metaInfo.group("Technical").
-                           item("Dimensions").value().toSize();
+                    KFileMetaInfo metaInfo(base + name);
+                    if (metaInfo.isValid())
+                    {
+                        if (metaInfo.containsGroup("Jpeg EXIF Data"))
+                        {
+                            dims = metaInfo.group("Jpeg EXIF Data").
+                              item("Dimensions").value().toSize();
+                        }
+                        else if (metaInfo.containsGroup("General"))
+                        {
+                            dims = metaInfo.group("General").
+                              item("Dimensions").value().toSize();
+                        }
+                        else if (metaInfo.containsGroup("Technical"))
+                        {
+                            dims = metaInfo.group("Technical").
+                              item("Dimensions").value().toSize();
+                        }                         
+                    }                    
                 }
             }
-        }
 
-        os << id;
-        os << albumid;
-        os << name;
-        os << date;
-        os << static_cast<size_t>(stbuf.st_size);
-        os << dims;
+            *os << id;
+            *os << albumid;
+            *os << name;
+            *os << date;
+            *os << static_cast<size_t>(stbuf.st_size);
+            *os << dims;
+
+            count++;
+
+            // Send images in batches of 200.
+            if (count > 200)
+            {
+                delete os;
+                os = 0;
+
+                SlaveBase::data(ba);
+                ba.resize(0);
+
+                count = 0;
+                os = new QDataStream(ba, IO_WriteOnly);
+            }
+        }
+        count++;
     }
 
     SlaveBase::data(ba);
@@ -234,6 +317,7 @@ static int write_all(int fd, const char *buf, size_t len)
 
 void kio_digikamalbums::get( const KURL& url )
 {
+// Code duplication from file:// ioslave
     kdDebug() << k_funcinfo << " : " << url << endl;    
 
     // get the libraryPath
@@ -319,6 +403,7 @@ void kio_digikamalbums::get( const KURL& url )
 
 void kio_digikamalbums::put(const KURL& url, int permissions, bool overwrite, bool /*resume*/)
 {
+// Code duplication from file:// ioslave
     kdDebug() << k_funcinfo << " : " << url.url() << endl;
 
     // get the libraryPath
@@ -442,6 +527,23 @@ void kio_digikamalbums::put(const KURL& url, int permissions, bool overwrite, bo
         }
     }
 
+    // set modification time
+    const QString mtimeStr = metaData( "modified" );
+    if ( !mtimeStr.isEmpty() ) {
+        QDateTime dt = QDateTime::fromString( mtimeStr, Qt::ISODate );
+        if ( dt.isValid() ) {
+            KDE_struct_stat dest_statbuf;
+            if (KDE_stat( _dest.data(), &dest_statbuf ) == 0) {
+                struct utimbuf utbuf;
+                utbuf.actime = dest_statbuf.st_atime; // access time, unchanged
+                utbuf.modtime = dt.toTime_t(); // modification time
+                kdDebug() << k_funcinfo << "setting modtime to " << utbuf.modtime << endl;
+                utime( _dest.data(), &utbuf );
+            }
+        }
+
+    }
+
     // First check if the file is already in database
     if (!findImage(album.id, url.fileName()))
     {
@@ -455,6 +557,7 @@ void kio_digikamalbums::put(const KURL& url, int permissions, bool overwrite, bo
 
 void kio_digikamalbums::copy( const KURL &src, const KURL &dst, int mode, bool overwrite )
 {
+// Code duplication from file:// ioslave?
     kdDebug() << k_funcinfo << "Src: " << src.path() << ", Dst: " << dst.path()   << endl;        
 
     // get the album library path
@@ -509,6 +612,7 @@ void kio_digikamalbums::copy( const KURL &src, const KURL &dst, int mode, bool o
     // metadata of the src album to the dst album
     if (src.fileName() == ".digikam_properties")
     {
+        // no duplication in AlbumDB?
         // copy metadata of album to destination album
         m_sqlDB.execSql( QString("UPDATE Albums SET date='%1', caption='%2', "
                                  "collection='%3', icon=%4 ")
@@ -698,6 +802,7 @@ void kio_digikamalbums::copy( const KURL &src, const KURL &dst, int mode, bool o
 
 void kio_digikamalbums::rename( const KURL& src, const KURL& dst, bool overwrite )
 {
+// Code duplication from file:// ioslave?
     kdDebug() << k_funcinfo << "Src: " << src << ", Dst: " << dst   << endl;        
 
     // if the filename is .digikam_properties fake that we renamed it
@@ -868,6 +973,7 @@ void kio_digikamalbums::stat( const KURL& url )
 
 void kio_digikamalbums::listDir( const KURL& url )
 {
+// Code duplication from file:// ioslave?
     kdDebug() << k_funcinfo << " : " << url.path() << endl;            
 
     QString libraryPath = url.user();
@@ -902,7 +1008,7 @@ void kio_digikamalbums::listDir( const KURL& url )
     listEntry(entry, false);
     while ((fi = it.current()) != 0)
     {
-        if (fi->fileName() != "." && fi->fileName() != "..")
+        if (fi->fileName() != "." && fi->fileName() != ".." || fi->extension(true) == "digikamtempfile.tmp")
         {
             createUDSEntry(fi->absFilePath(), entry);
             listEntry(entry, false);
@@ -917,6 +1023,7 @@ void kio_digikamalbums::listDir( const KURL& url )
 
 void kio_digikamalbums::mkdir( const KURL& url, int permissions )
 {
+// Code duplication from file:// ioslave?
     kdDebug() << k_funcinfo << " : " << url.url() << endl;
 
     QString libraryPath = url.user();
@@ -959,6 +1066,7 @@ void kio_digikamalbums::mkdir( const KURL& url, int permissions )
         }
         else
         {
+            // code similar to AlbumDB::addAlbum
             m_sqlDB.execSql( QString("REPLACE INTO Albums (url, date) "
                                      "VALUES('%1','%2')")
                              .arg(escapeString(url.path()),
@@ -988,7 +1096,8 @@ void kio_digikamalbums::mkdir( const KURL& url, int permissions )
 
 void kio_digikamalbums::chmod( const KURL& url, int permissions )
 {
-    kdDebug() << k_funcinfo << " : " << url.url() << endl;            
+// Code duplication from file:// ioslave?
+    kdDebug() << k_funcinfo << " : " << url.url() << endl;
 
     // get the album library path
     QString libraryPath = url.user();
@@ -1007,6 +1116,7 @@ void kio_digikamalbums::chmod( const KURL& url, int permissions )
 
 void kio_digikamalbums::del( const KURL& url, bool isfile)
 {
+// Code duplication from file:// ioslave?
     kdDebug() << k_funcinfo << " : " << url.url() << endl;            
 
     // get the album library path
@@ -1136,7 +1246,20 @@ bool kio_digikamalbums::createUDSEntry(const QString& path, KIO::UDSEntry& entry
     atom.m_uds = KIO::UDS_NAME;
     atom.m_str = QFileInfo(path).fileName();
     entry.append(atom);
-    
+
+    /*
+    // If we provide the local path, a KIO::CopyJob will optimize away
+    // the use of our custom digikamalbums:/ ioslave, which breaks
+    // copying the database entry:
+    // Disabling this as a temporary solution for bug #137282
+    // This code is intended as a fix for bug #122653.
+#if KDE_IS_VERSION(3,4,0)
+    atom.m_uds = KIO::UDS_LOCAL_PATH;
+    atom.m_str = path;
+    entry.append(atom);
+#endif
+    */
+
     return true;
 }
 
@@ -1173,6 +1296,7 @@ void kio_digikamalbums::createDigikamPropsUDSEntry(KIO::UDSEntry& entry)
 
 void kio_digikamalbums::buildAlbumList()
 {
+// simplified from AlbumDB::scanAlbums()
     m_albumList.clear();
     
     QStringList values;
@@ -1202,6 +1326,7 @@ void kio_digikamalbums::buildAlbumList()
 
 AlbumInfo kio_digikamalbums::findAlbum(const QString& url, bool addIfNotExists)
 {
+// similar to AlbumDB::getOrCreateAlbumId
     AlbumInfo album;
     for (QValueList<AlbumInfo>::const_iterator it = m_albumList.begin();
          it != m_albumList.end(); ++it)
@@ -1239,12 +1364,14 @@ AlbumInfo kio_digikamalbums::findAlbum(const QString& url, bool addIfNotExists)
 
 void kio_digikamalbums::delAlbum(int albumID)
 {
+// code duplication from AlbumDB::deleteAlbum
     m_sqlDB.execSql(QString("DELETE FROM Albums WHERE id='%1'")
                     .arg(albumID));    
 }
 
 void kio_digikamalbums::renameAlbum(const QString& oldURL, const QString& newURL)
 {
+// similar to AlbumDB::setAlbumURL, but why more extended?
     // first update the url of the album which was renamed
 
     m_sqlDB.execSql( QString("UPDATE Albums SET url='%1' WHERE url='%2'")
@@ -1270,6 +1397,7 @@ void kio_digikamalbums::renameAlbum(const QString& oldURL, const QString& newURL
 
 bool kio_digikamalbums::findImage(int albumID, const QString& name) const
 {
+// no similar method in AlbumDB?
     QStringList values;
     
     m_sqlDB.execSql( QString("SELECT name FROM Images "
@@ -1281,12 +1409,46 @@ bool kio_digikamalbums::findImage(int albumID, const QString& name) const
     return !(values.isEmpty());
 }
 
+// from albuminfo.h
+class TagInfo
+{
+public:
+
+    typedef QValueList<TagInfo> List;
+
+    int      id;
+    int      pid;
+    QString  name;
+    QString  icon;
+};
+
 void kio_digikamalbums::addImage(int albumID, const QString& filePath)
 {
+// Code duplication: ScanLib::storeItemInDatabase, AlbumDB::addItem,
+//                   AlbumDB::setItemRating, AlbumDB::addItemTag, AlbumDB::addTag
+
+    // from ScanLib::storeItemInDatabase
     QString   comment;
     QDateTime datetime;
+    int       rating = 0;
 
-    Digikam::readJPEGMetaData(filePath, comment, datetime);
+    Digikam::DMetadata metadata(filePath);
+
+    // Trying to get comments from image :
+    // In first, from standard JPEG comments, or
+    // In second, from EXIF comments tag, or
+    // In third, from IPTC comments tag.
+
+    comment = metadata.getImageComment();
+
+    // Trying to get date and time from image :
+    // In first, from EXIF date & time tags, or
+    // In second, from IPTC date & time tags.
+
+    datetime = metadata.getImageDateTime();
+
+    // Trying to get image rating from IPTC Urgency tag.
+    rating = metadata.getImageRating();
 
     if (!datetime.isValid())
     {
@@ -1294,6 +1456,10 @@ void kio_digikamalbums::addImage(int albumID, const QString& filePath)
         datetime = info.lastModified();
     }
 
+    // Try to get image tags from IPTC keywords tags.
+    QStringList keywordsList = metadata.getImageKeywords();
+
+    // from AlbumDB::addItem
     m_sqlDB.execSql(QString("REPLACE INTO Images "
                             "(dirid, name, datetime, caption) "
                             "VALUES(%1, '%2', '%3', '%4')")
@@ -1301,10 +1467,202 @@ void kio_digikamalbums::addImage(int albumID, const QString& filePath)
                          escapeString(QFileInfo(filePath).fileName()),
                          datetime.toString(Qt::ISODate),
                          escapeString(comment)));
+
+    Q_LLONG imageID = m_sqlDB.lastInsertedRow();
+
+    // from AlbumDB::setItemRating
+    if (imageID != -1 && rating != -1)
+    {
+        m_sqlDB.execSql(QString("REPLACE INTO ImageProperties "
+                                "(imageid, property, value) "
+                                "VALUES(%1, '%2', '%3');")
+                        .arg(imageID)
+                        .arg("Rating")
+                        .arg(rating) );
+    }
+
+    // Set existing tags in database or create new tags if not exist.
+
+    if ( imageID != -1 && !keywordsList.isEmpty() )
+    {
+        QStringList keywordsList2Create;
+
+        // Create a list of the tags currently in database
+
+        TagInfo::List tagsList;
+
+        QStringList values;
+        m_sqlDB.execSql( "SELECT id, pid, name FROM Tags;", &values );
+
+        for (QStringList::iterator it = values.begin(); it != values.end();)
+        {
+            TagInfo info;
+
+            info.id   = (*it).toInt();
+            ++it;
+            info.pid  = (*it).toInt();
+            ++it;
+            info.name = *it;
+            ++it;
+            tagsList.append(info);
+        }
+
+        // For every tag in keywordsList, scan taglist to check if tag already exists.
+
+        for (QStringList::iterator kwd = keywordsList.begin();
+            kwd != keywordsList.end(); ++kwd )
+        {
+            // split full tag "url" into list of single tag names 
+            QStringList tagHierarchy = QStringList::split('/', *kwd);
+            if (tagHierarchy.isEmpty())
+                continue;
+
+            // last entry in list is the actual tag name
+            bool foundTag   = false;
+            QString tagName = tagHierarchy.back();
+            tagHierarchy.pop_back();
+
+            for (TagInfo::List::iterator tag = tagsList.begin();
+                tag != tagsList.end(); ++tag )
+            {
+                // There might be multiple tags with the same name, but in different
+                // hierarchies. We must check them all until we find the correct hierarchy
+                if ((*tag).name == tagName)
+                {
+                    int parentID = (*tag).pid;
+
+                    // Check hierarchy, from bottom to top
+                    bool foundParentTag                 = true;
+                    QStringList::iterator parentTagName = tagHierarchy.end();
+
+                    while (foundParentTag && parentTagName != tagHierarchy.begin())
+                    {
+                        --parentTagName;
+
+                        foundParentTag = false;
+
+                        for (TagInfo::List::iterator parentTag = tagsList.begin();
+                            parentTag != tagsList.end(); ++parentTag )
+                        {
+                            // check if name is the same, and if ID is identical
+                            // to the parent ID we got from the child tag
+                            if ( (*parentTag).id == parentID &&
+                                (*parentTag).name == (*parentTagName) )
+                            {
+                                parentID       = (*parentTag).pid;
+                                foundParentTag = true;
+                                break;
+                            }
+                        }
+
+                        // If we traversed the list without a match,
+                        // foundParentTag will be false, the while loop breaks.
+                    }
+
+                    // If we managed to traverse the full hierarchy,
+                    // we have our tag.
+                    if (foundParentTag)
+                    {
+                        // from AlbumDB::addItemTag
+                        m_sqlDB.execSql( QString("REPLACE INTO ImageTags (imageid, tagid) "
+                                                 "VALUES(%1, %2);")
+                                         .arg(imageID)
+                                         .arg((*tag).id) );
+                        foundTag = true;
+                        break;
+                    }
+                }
+            }
+
+            if (!foundTag)
+                keywordsList2Create.append(*kwd);
+        }
+
+        // If tags do not exist in database, create them.
+
+        if (!keywordsList2Create.isEmpty())
+        {
+            for (QStringList::iterator kwd = keywordsList2Create.begin(); 
+                kwd != keywordsList2Create.end(); ++kwd )
+            {
+                // split full tag "url" into list of single tag names 
+                QStringList tagHierarchy = QStringList::split('/', *kwd);
+
+                if (tagHierarchy.isEmpty())
+                    continue;
+
+                int  parentTagID      = 0;
+                int  tagID            = 0;
+                bool parentTagExisted = true;
+
+                // Traverse hierarchy from top to bottom
+                for (QStringList::iterator tagName = tagHierarchy.begin();
+                    tagName != tagHierarchy.end(); ++tagName)
+                {
+                    tagID = 0;
+
+                    // if the parent tag did not exist, we need not check if the child exists
+                    if (parentTagExisted)
+                    {
+                        for (TagInfo::List::iterator tag = tagsList.begin();
+                            tag != tagsList.end(); ++tag )
+                        {
+                            // find the tag with tag name according to tagHierarchy,
+                            // and parent ID identical to the ID of the tag we found in
+                            // the previous run.
+                            if ((*tag).name == (*tagName) && (*tag).pid == parentTagID)
+                            {
+                                tagID = (*tag).id;
+                                break;
+                            }
+                        }
+                    }
+
+                    if (tagID != 0)
+                    {
+                        // tag already found in DB
+                        parentTagID = tagID;
+                        continue;
+                    }
+
+                    // Tag does not yet exist in DB, add it
+                    // from AlbumDB::addTag
+                    m_sqlDB.execSql( QString("INSERT INTO Tags (pid, name, icon) "
+                                             "VALUES( %1, '%2', 0)")
+                                     .arg(parentTagID)
+                                     .arg(escapeString(*tagName)));
+                    tagID = m_sqlDB.lastInsertedRow();
+
+                    if (tagID == -1)
+                    {
+                        // Something is wrong in database. Abort.
+                        break;
+                    }
+
+                    // append to our list of existing tags (for following keywords)
+                    TagInfo info;
+                    info.id   = tagID;
+                    info.pid  = parentTagID;
+                    info.name = (*tagName);
+                    tagsList.append(info);
+
+                    parentTagID      = tagID;
+                    parentTagExisted = false;
+                }
+
+                // from AlbumDB::addItemTag
+                m_sqlDB.execSql( QString("REPLACE INTO ImageTags (imageid, tagid) "
+                                         "VALUES(%1, %2);")
+                                 .arg(imageID)
+                                 .arg(tagID) );
+            }
+        }
+    }
 }
 
 void kio_digikamalbums::delImage(int albumID, const QString& name)
 {
+// code duplication from AlbumDB::deleteItem
     m_sqlDB.execSql( QString("DELETE FROM Images "
                              "WHERE dirid=%1 AND name='%2';")
                      .arg(albumID)
@@ -1314,6 +1672,7 @@ void kio_digikamalbums::delImage(int albumID, const QString& name)
 void kio_digikamalbums::renameImage(int oldAlbumID, const QString& oldName,
                                     int newAlbumID, const QString& newName)
 {
+// code duplication from AlbumDB::deleteItem, AlbumDB::moveItem
     // first delete any stale entries for the destination file
     m_sqlDB.execSql( QString("DELETE FROM Images "
                              "WHERE dirid=%1 AND name='%2';")
@@ -1332,30 +1691,55 @@ void kio_digikamalbums::renameImage(int oldAlbumID, const QString& oldName,
 void kio_digikamalbums::copyImage(int srcAlbumID, const QString& srcName,
                                   int dstAlbumID, const QString& dstName)
 {
+// code duplication from AlbumDB::copyItem
+    // check for src == dest
+    if (srcAlbumID == dstAlbumID && srcName == dstName)
+    {
+        error( KIO::ERR_FILE_ALREADY_EXIST, dstName );
+        return;
+    }
+
+    // find id of src image
+    QStringList values;
+    m_sqlDB.execSql( QString("SELECT id FROM Images "
+                             "WHERE dirid=%1 AND name='%2';")
+                     .arg(QString::number(srcAlbumID), escapeString(srcName)),
+                     &values);
+
+    if (values.isEmpty())
+    {
+        error(KIO::ERR_UNKNOWN, i18n("Source image %1 not found in database")
+                .arg(srcName));
+        return;
+    }
+
+    int srcId = values[0].toInt();
+
     // first delete any stale entries for the destination file
     m_sqlDB.execSql( QString("DELETE FROM Images "
                              "WHERE dirid=%1 AND name='%2';")
-                     .arg(dstAlbumID)
-                     .arg(escapeString(dstName)) );
+                     .arg(QString::number(dstAlbumID), escapeString(dstName)) );
 
-    // now copy the metadata over
+    // copy entry in Images table
     m_sqlDB.execSql( QString("INSERT INTO Images (dirid, name, caption, datetime) "
                              "SELECT %1, '%2', caption, datetime FROM Images "
-                             "WHERE dirid=%3 AND name='%4';")
-                     .arg(QString::number(dstAlbumID),
-                          escapeString(dstName),
-                          QString::number(srcAlbumID),
-                          escapeString(srcName)) );
+                             "WHERE id=%3;")
+                     .arg(QString::number(dstAlbumID), escapeString(dstName),
+                          QString::number(srcId)) );
 
+    int dstId = m_sqlDB.lastInsertedRow();
+
+    // copy tags
     m_sqlDB.execSql( QString("INSERT INTO ImageTags (imageid, tagid) "
-                             "SELECT A.id, B.tagid FROM Images AS A, ImageTags AS B "
-                             "WHERE A.dirid = %1 AND A.name = '%2' AND"
-                             "      B.imageid = (SELECT id FROM Images "
-                             "                   WHERE dirid=%3 AND name='%4')")
-                     .arg(QString::number(dstAlbumID),
-                          escapeString(dstName),
-                          QString::number(srcAlbumID),
-                          escapeString(srcName)) );
+                             "SELECT %1, tagid FROM ImageTags "
+                             "WHERE imageid=%2;")
+                     .arg(QString::number(dstId), QString::number(srcId)) );
+
+    // copy properties (rating)
+    m_sqlDB.execSql( QString("INSERT INTO ImageProperties (imageid, property, value) "
+                             "SELECT %1, property, value FROM ImageProperties "
+                             "WHERE imageid=%2;")
+                     .arg(QString::number(dstId), QString::number(srcId)) );
 }
 
 void kio_digikamalbums::scanAlbum(const QString& url)
@@ -1374,7 +1758,7 @@ void kio_digikamalbums::scanOneAlbum(const QString& url)
 
     QString subURL = url;
     if (!url.endsWith("/"))
-        subURL += "/";
+        subURL += '/';
     subURL = escapeString( subURL);
 
     {
@@ -1404,7 +1788,7 @@ void kio_digikamalbums::scanOneAlbum(const QString& url)
                 continue;
             }
             
-            QString u = QDir::cleanDirPath(url + "/" + fi->fileName());
+            QString u = QDir::cleanDirPath(url + '/' + fi->fileName());
             
             if (currAlbumList.contains(u))
             {
@@ -1458,13 +1842,19 @@ void kio_digikamalbums::scanOneAlbum(const QString& url)
         {
             ++it;
 
+            // ignore temp files we created ourselves
+            if (fi->extension(true) == "digikamtempfile.tmp")
+            {
+                continue;
+            }
+
             if (currItemList.contains(fi->fileName()))
             {
                 currItemList.remove(fi->fileName());
                 continue;
             }
 
-            addImage(albumID, m_libraryPath + url + "/" + fi->fileName());
+            addImage(albumID, m_libraryPath + url + '/' + fi->fileName());
         }
 
         // currItemList now contains deleted file list. remove them from db
