@@ -1,8 +1,12 @@
 /* ============================================================
- * File  : digikamsearch.cpp
- * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- * Date  : 2005-04-21
- * Description :
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ * 
+ * Date        : 2005-04-21
+ * Description : a kio-slave to process search on digiKam albums
+ *
+ * Copyright (C) 2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -17,7 +21,36 @@
  *
  * ============================================================ */
 
-#include <digikam_export.h>
+// C Ansi includes.
+
+extern "C"
+{
+#include <unistd.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/time.h>
+#include <utime.h>
+}
+
+// C++ includes.
+
+#include <cerrno>
+#include <cstdlib>
+#include <cstdio>
+#include <ctime>
+
+// Qt includes.
+
+#include <qfile.h>
+#include <qdatastream.h>
+#include <qtextstream.h>
+#include <qregexp.h>
+#include <qdir.h>
+#include <qvariant.h>
+#include <qmap.h>
+
+// KDE includes.
 
 #include <kglobal.h>
 #include <klocale.h>
@@ -30,33 +63,14 @@
 #include <kio/ioslave_defaults.h>
 #include <klargefile.h>
 
-#include <qfile.h>
-#include <qdatastream.h>
-#include <qtextstream.h>
-#include <qregexp.h>
-#include <qdir.h>
-#include <qvariant.h>
-#include <qmap.h>
-
-extern "C"
-{
-#include <stdlib.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#include <time.h>
-#include <utime.h>
-}
+// Local includes.
 
+#include "digikam_export.h"
 #include "digikamsearch.h"
 
 kio_digikamsearch::kio_digikamsearch(const QCString &pool_socket,
                                      const QCString &app_socket)
-    : SlaveBase("kio_digikamsearch", pool_socket, app_socket)
+                 : SlaveBase("kio_digikamsearch", pool_socket, app_socket)
 {
     // build a lookup table for month names
     const KCalendarSystem* cal = KGlobal::locale()->calendar();
@@ -65,7 +79,6 @@ kio_digikamsearch::kio_digikamsearch(const QCString &pool_socket,
         m_shortMonths[i-1] = cal->monthName(i, 2000, true).lower();
         m_longMonths[i-1]  = cal->monthName(i, 2000, false).lower();
     }
-
 }
 
 kio_digikamsearch::~kio_digikamsearch()
@@ -85,7 +98,8 @@ static QValueList<QRegExp> makeFilterList( const QString &filter )
 
     QStringList list = QStringList::split( sep, filter );
     QStringList::Iterator it = list.begin();
-    while ( it != list.end() ) {
+    while ( it != list.end() ) 
+    {
         regExps << QRegExp( (*it).stripWhiteSpace(), false, true );
         ++it;
     }
@@ -96,7 +110,8 @@ static bool matchFilterList( const QValueList<QRegExp>& filters,
                              const QString &fileName )
 {
     QValueList<QRegExp>::ConstIterator rit = filters.begin();
-    while ( rit != filters.end() ) {
+    while ( rit != filters.end() ) 
+    {
         if ( (*rit).exactMatch(fileName) )
             return true;
         ++rit;
@@ -111,12 +126,17 @@ void kio_digikamsearch::special(const QByteArray& data)
     QString filter;
     int     getDimensions;
     int     listingType = 0;
+    int     recurseAlbums;
+    int     recurseTags;
 
     QDataStream ds(data, IO_ReadOnly);
     ds >> libraryPath;
     ds >> url;
     ds >> filter;
     ds >> getDimensions;
+    ds >> recurseAlbums;
+    ds >> recurseTags;
+
     if (!ds.atEnd())
         ds >> listingType;
 
@@ -182,7 +202,7 @@ void kio_digikamsearch::special(const QByteArray& data)
             if (!matchFilterList(regex, name))
                 continue;
 
-            path = m_libraryPath + purl + "/" + name;
+            path = m_libraryPath + purl + '/' + name;
             if (::stat(QFile::encodeName(path), &stbuf) != 0)
                 continue;
 
@@ -271,7 +291,6 @@ void kio_digikamsearch::special(const QByteArray& data)
 
     SlaveBase::data(ba);
 
-
     finished();
 }
 
@@ -347,8 +366,12 @@ QString kio_digikamsearch::buildQuery(const KURL& url) const
             rule.op = NE;
         else if (op == "lt")
             rule.op = LT;
+        else if (op == "lte")
+            rule.op = LTE;
         else if (op == "gt")
             rule.op = GT;
+        else if (op == "gte")
+            rule.op = GTE;
         else if (op == "like")
             rule.op = LIKE;
         else if (op == "nlike")
@@ -405,7 +428,7 @@ QString kio_digikamsearch::buildQuery(const KURL& url) const
                     todo.append( IMAGECAPTION );
                     todo.append( RATING );
 
-                    sqlQuery += "(";
+                    sqlQuery += '(';
                     QValueListIterator<SKey> it;
                     it = todo.begin();
                     while ( it != todo.end() )
@@ -415,7 +438,7 @@ QString kio_digikamsearch::buildQuery(const KURL& url) const
                         if ( it != todo.end() )
                             sqlQuery += " OR ";
                     }
-                    sqlQuery += ")";
+                    sqlQuery += ')';
                 }
             }
             else
@@ -425,7 +448,7 @@ QString kio_digikamsearch::buildQuery(const KURL& url) const
         }
         else
         {
-            sqlQuery += " " + *it + " ";
+            sqlQuery += ' ' + *it + ' ';
         }
     }
 
@@ -440,124 +463,166 @@ QString kio_digikamsearch::subQuery(enum kio_digikamsearch::SKey key,
 
     switch (key)
     {
-    case(ALBUM):
-    {
-        query = " (Images.dirid $$##$$ $$@@$$) ";
-        break;
-    }
-    case(ALBUMNAME):
-    {
-        query = " (Images.dirid IN "
-                "  (SELECT id FROM Albums WHERE url $$##$$ $$@@$$)) ";
-        break;
-    }
-    case(ALBUMCAPTION):
-    {
-        query = " (Images.dirid IN "
-                "  (SELECT id FROM Albums WHERE caption $$##$$ $$@@$$)) ";
-        break;
-    }
-    case(ALBUMCOLLECTION):
-    {
-        query = " (Images.dirid IN "
-                "  (SELECT id FROM Albums WHERE collection $$##$$ $$@@$$)) ";
-        break;
-    }
-    case(TAG):
-    {
-        if (op == EQ)
+        case(ALBUM):
+        {
+            query = " (Images.dirid $$##$$ $$@@$$) ";
+            break;
+        }
+        case(ALBUMNAME):
+        {
+            query = " (Images.dirid IN "
+                    "  (SELECT id FROM Albums WHERE url $$##$$ $$@@$$)) ";
+            break;
+        }
+        case(ALBUMCAPTION):
+        {
+            query = " (Images.dirid IN "
+                    "  (SELECT id FROM Albums WHERE caption $$##$$ $$@@$$)) ";
+            break;
+        }
+        case(ALBUMCOLLECTION):
+        {
+            query = " (Images.dirid IN "
+                    "  (SELECT id FROM Albums WHERE collection $$##$$ $$@@$$)) ";
+            break;
+        }
+        case(TAG):
+        {
+            if (op == EQ)
+                query = " (Images.id IN "
+                        "   (SELECT imageid FROM ImageTags "
+                        "    WHERE tagid = $$@@$$)) ";
+            else if (op == NE)
+                query = " (Images.id NOT IN "
+                        "   (SELECT imageid FROM ImageTags "
+                        "    WHERE tagid = $$@@$$)) ";
+            else if (op == LIKE) 
+                query = " (Images.id IN "
+                        "   (SELECT ImageTags.imageid FROM ImageTags JOIN TagsTree on ImageTags.tagid = TagsTree.id "
+                        "    WHERE TagsTree.pid = $$@@$$ or ImageTags.tagid = $$@@$$ )) ";
+            else // op == NLIKE
+                query = " (Images.id NOT IN "
+                        "   (SELECT ImageTags.imageid FROM ImageTags JOIN TagsTree on ImageTags.tagid = TagsTree.id "
+                        "    WHERE TagsTree.pid = $$@@$$ or ImageTags.tagid = $$@@$$ )) ";
+    
+    //         query = " (Images.id IN "
+    //                 "   (SELECT imageid FROM ImageTags "
+    //                 "    WHERE tagid $$##$$ $$@@$$)) ";
+    
+            query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                        + QString::fromLatin1("'"));
+    
+            break;
+        }
+        case(TAGNAME):
+        {
             query = " (Images.id IN "
-                    "   (SELECT imageid FROM ImageTags "
-                    "    WHERE tagid = $$@@$$)) ";
-        else 
-            query = " (Images.id NOT IN "
-                    "   (SELECT imageid FROM ImageTags "
-                    "    WHERE tagid = $$@@$$)) ";
+                    "  (SELECT imageid FROM ImageTags "
+                    "   WHERE tagid IN "
+                    "   (SELECT id FROM Tags WHERE name $$##$$ $$@@$$))) ";
+            break;
+        }
+        case(IMAGENAME):
+        {
+            query = " (Images.name $$##$$ $$@@$$) ";
+            break;
+        }
+        case(IMAGECAPTION):
+        {
+            query = " (Images.caption $$##$$ $$@@$$) ";
+            break;
+        }
+        case(IMAGEDATE):
+        {
+            query = " (Images.datetime $$##$$ $$@@$$) ";
+            break;
+        }
+        case (KEYWORD):
+        {
+            kdWarning() << "KEYWORD Detected which is not possible" << endl;
+            break;
+        }
+        case(RATING):
+        {   
+            // For searches for `rating=0`, `rating>=0`, `rating<=0`, 
+            // `rating <c`, `rating<=c`, `rating<>c` with c=1,2,3,4,5, 
+            // special care has to be taken: Images which were never rated  
+            // have no ImageProperties.property='Rating', but 
+            // need to be treated like having a rating of 0.
+            // This is achieved by including all images which do
+            // not have property='Rating'.
+            if ( (   val=="0" and (op==EQ or op==GTE or op==LTE) )
+                 or (val!="0" and (op==LT or op==LTE or op==NE) ) ) {
+                query = " ( (ImageProperties.value $$##$$ $$@@$$ and ImageProperties.property='Rating') or (Images.id NOT IN (SELECT imageid FROM ImageProperties WHERE property='Rating') ) )";
+            } else {           
+                query = " (ImageProperties.value $$##$$ $$@@$$ and ImageProperties.property='Rating') ";
+            }
+            break;
+        }
 
-//         query = " (Images.id IN "
-//                 "   (SELECT imageid FROM ImageTags "
-//                 "    WHERE tagid $$##$$ $$@@$$)) ";
-        break;
     }
-    case(TAGNAME):
+    
+    if (key != TAG) 
     {
-        query = " (Images.id IN "
-                "  (SELECT imageid FROM ImageTags "
-                "   WHERE tagid IN "
-                "   (SELECT id FROM Tags WHERE name $$##$$ $$@@$$))) ";
-        break;
-    }
-    case(IMAGENAME):
-    {
-        query = " (Images.name $$##$$ $$@@$$) ";
-        break;
-    }
-    case(IMAGECAPTION):
-    {
-        query = " (Images.caption $$##$$ $$@@$$) ";
-        break;
-    }
-    case(IMAGEDATE):
-    {
-        query = " (Images.datetime $$##$$ $$@@$$) ";
-        break;
-    }
-    case (KEYWORD):
-    {
-        kdWarning() << "KEYWORD Detected which is not possible" << endl;
-        break;
-    }
-    case(RATING):
-    {
-        query = " (ImageProperties.value $$##$$ $$@@$$ and ImageProperties.property='Rating') ";
-        break;
-    }
-    }
-
-    switch (op)
-    {
-    case(EQ):
-    {
-        query.replace("$$##$$", "=");
-        query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
-                      + QString::fromLatin1("'"));
-        break;
-    }
-    case(NE):
-    {
-        query.replace("$$##$$", "<>");
-        query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
-                      + QString::fromLatin1("'"));
-        break;
-    }
-    case(LT):
-    {
-        query.replace("$$##$$", "<");
-        query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
-                      + QString::fromLatin1("'"));
-        break;
-    }
-    case(GT):
-    {
-        query.replace("$$##$$", ">");
-        query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
-                      + QString::fromLatin1("'"));
-        break;
-    }
-    case(LIKE):
-    {
-        query.replace("$$##$$", "LIKE");
-        query.replace("$$@@$$", QString::fromLatin1("'%") + escapeString(val)
-                      + QString::fromLatin1("%'"));
-        break;
-    }
-    case(NLIKE):
-    {
-        query.replace("$$##$$", "NOT LIKE");
-        query.replace("$$@@$$", QString::fromLatin1("'%") + escapeString(val)
-                      + QString::fromLatin1("%'"));
-        break;
-    }
+        switch (op)
+        {
+            case(EQ):
+            {
+                query.replace("$$##$$", "=");
+                query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                            + QString::fromLatin1("'"));
+                break;
+            }
+            case(NE):
+            {
+                query.replace("$$##$$", "<>");
+                query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                            + QString::fromLatin1("'"));
+                break;
+            }
+            case(LT):
+            {
+                query.replace("$$##$$", "<");
+                query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                            + QString::fromLatin1("'"));
+                break;
+            }
+            case(GT):
+            {
+                query.replace("$$##$$", ">");
+                query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                            + QString::fromLatin1("'"));
+                break;
+            }
+            case(LTE):
+            {
+                query.replace("$$##$$", "<=");
+                query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                            + QString::fromLatin1("'"));
+                break;
+            }
+            case(GTE):
+            {
+                query.replace("$$##$$", ">=");
+                query.replace("$$@@$$", QString::fromLatin1("'") + escapeString(val)
+                            + QString::fromLatin1("'"));
+                break;
+            }
+            case(LIKE):
+            {
+                query.replace("$$##$$", "LIKE");
+                query.replace("$$@@$$", QString::fromLatin1("'%") + escapeString(val)
+                            + QString::fromLatin1("%'"));
+                break;
+            }
+            case(NLIKE):
+            {
+                query.replace("$$##$$", "NOT LIKE");
+                query.replace("$$@@$$", QString::fromLatin1("'%") + escapeString(val)
+                            + QString::fromLatin1("%'"));
+                break;
+            }
+        }
     }
 
     // special case for imagedate. If the key is imagedate and the operator is EQ,
@@ -586,7 +651,8 @@ extern "C"
         KInstance instance( "kio_digikamsearch" );
         KGlobal::locale();
 
-        if (argc != 4) {
+        if (argc != 4) 
+        {
             kdDebug() << "Usage: kio_digikamsearch  protocol domain-socket1 domain-socket2"
                       << endl;
             exit(-1);
@@ -637,5 +703,5 @@ QString kio_digikamsearch::possibleDate(const QString& str, bool& exact) const
         }
     }
 
-    return QString::null;
+    return QString();
 }
