@@ -1,12 +1,17 @@
 /* ============================================================
- * Authors: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- *          Gilles Caulier <caulier dot gilles at kdemail dot net> 
- * Date   : 2003-01-15
- * Description : digiKam KIO slave to get image thumbnails.
  *
- * Copyright 2003-2005 by Renchi Raju, Gilles Caulier
- * Copyright 2006      by Gilles Caulier
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ * 
+ * Date        : 2003-01-15
+ * Description : digiKam KIO slave to get image thumbnails. 
+ *               This kio-slave support this freedesktop 
+ *               specification about thumbnails mamagement:
+ *               http://jens.triq.net/thumbnail-spec
  *
+ * Copyright (C) 2003-2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
+ * Copyright (C) 2003-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * 
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
@@ -48,7 +53,6 @@
 #include <kdebug.h>
 #include <kurl.h>
 #include <kinstance.h>
-#include <kio/global.h>
 #include <kimageio.h>
 #include <klocale.h>
 #include <kglobal.h>
@@ -59,18 +63,28 @@
 #include <klibloader.h>
 #include <kmimetype.h>
 #include <kprocess.h>
+#include <kio/global.h>
 #include <kio/thumbcreator.h>
+#include <kfilemetainfo.h>
 
-// Lib KExif includes.
+// LibKDcraw includes.
 
-#include <libkexif/kexifdata.h>
+#include <libkdcraw/rawfiles.h>
+#include <libkdcraw/kdcraw.h>
+
+// Local includes
+
+#include "dimg.h"
+#include "dmetadata.h"
+#include "jpegutils.h"
+#include "digikamthumbnail.h"
+#include "digikam_export.h"
 
 // C Ansi includes.
 
 extern "C"
 {
 #include <unistd.h>
-#include <jpeglib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/ipc.h>
@@ -79,18 +93,8 @@ extern "C"
 #include <png.h>
 }
 
-// Imlib2 includes.
-
-#define X_DISPLAY_MISSING 1
-#include <Imlib2.h>
-
-// Local includes
-
-#include "rawfiles.h"
-#include "digikamthumbnail.h"
-#include "digikam_export.h"
-
 using namespace KIO;
+using namespace Digikam;
 
 kio_digikamthumbnailProtocol::kio_digikamthumbnailProtocol(int argc, char** argv) 
                             : SlaveBase("kio_digikamthumbnail", argv[2], argv[3])
@@ -98,7 +102,7 @@ kio_digikamthumbnailProtocol::kio_digikamthumbnailProtocol(int argc, char** argv
     argc_ = argc;
     argv_ = argv;
     app_  = 0;
-    
+
     createThumbnailDirs();
 }
 
@@ -139,7 +143,7 @@ void kio_digikamthumbnailProtocol::get(const KURL& url )
 
     // NOTE: if thumbnail have not been generated by digiKam (konqueror for example),
     //       force to recompute it, else we use it.
-    
+
     img = loadPNG(thumbPath);
     if (!img.isNull())
     {
@@ -150,30 +154,25 @@ void kio_digikamthumbnailProtocol::get(const KURL& url )
 
     if (regenerate)
     {
-        // In first we trying to load image using the file extension. This is mandatory because
-        // some tiff files are detected like RAW files by dcraw::parse method.
+        // To speed-up thumb extraction, we trying to load image using the file extension.
         if ( !loadByExtension(img, url.path()) )
         {
-            // Try JPEG loading...
+            // Try JPEG loading : JPEG files without using Exif Thumb.
             if ( !loadJPEG(img, url.path()) )
             {
                 // Try to load with dcraw : RAW files.
-                if ( !loadDCRAW(img, url.path()) )
+                if (!KDcrawIface::KDcraw::loadDcrawPreview(img, url.path()) )
                 {
-                    // Try to load with imlib2 : TIFF, PNG, etc.
-                    if ( !loadImlib2(img, url.path()) )
+                    // Try to load with DImg : TIFF, PNG, etc.
+                    if (!loadDImg(img, url.path()) )
                     {
-                        // Try to load with QT/KDELib
-                        if ( !img.load(url.path()) )
-                        {
-                            // Try to load with KDE thumbcreators : video files and others stuff.
-                            loadKDEThumbCreator(img, url.path());
-                        }
+                        // Try to load with KDE thumbcreators : video files and others stuff.
+                        loadKDEThumbCreator(img, url.path());
                     }
                 }
             }
         }
-        
+
         if (img.isNull())
         {
             error(KIO::ERR_INTERNAL, i18n("Cannot create thumbnail for %1")
@@ -187,9 +186,9 @@ void kio_digikamthumbnailProtocol::get(const KURL& url )
 
         if (img.depth() != 32)
             img = img.convertDepth(32);
-            
+
         if (exif)
-            exifRotate(url.path(), img);            
+            exifRotate(url.path(), img);
 
         img.setText(QString("Thumb::URI").latin1(),   0, uri);
         img.setText(QString("Thumb::MTime").latin1(), 0, QString::number(st.st_mtime));
@@ -215,8 +214,7 @@ void kio_digikamthumbnailProtocol::get(const KURL& url )
     QByteArray imgData;
     QDataStream stream( imgData, IO_WriteOnly );
 
-    QString shmid = metaData("shmid");
-
+    const QString shmid = metaData("shmid");
     if (shmid.isEmpty())
     {
         stream << img;
@@ -254,224 +252,92 @@ bool kio_digikamthumbnailProtocol::loadByExtension(QImage& image, const QString&
     QFileInfo fileInfo(path);
     if (!fileInfo.exists())
         return false;
-    
-    QString ext = fileInfo.extension().upper();
 
-    if (ext == QString("JPEG") || ext == QString("JPG"))
-        return (loadJPEG(image, path));
-    else if (ext == QString("PNG"))
-        return (image.load(path));
-    else if (ext == QString("TIFF") || ext == QString("TIF"))
-        return (image.load(path));
-    
-    return false;
-}
-
-// -- JPEG Extraction ---------------------------------------------------------------------
-
-struct myjpeg_error_mgr : public jpeg_error_mgr
-{
-    jmp_buf setjmp_buffer;
-};
-
-extern "C"
-{
-    static void myjpeg_error_exit(j_common_ptr cinfo)
+    // Try to use embedded preview image from metadata.    
+    DMetadata metadata(path);
+    if (metadata.getImagePreview(image))
     {
-        myjpeg_error_mgr* myerr = (myjpeg_error_mgr*) cinfo->err;
-
-        char buffer[JMSG_LENGTH_MAX];
-        (*cinfo->err->format_message)(cinfo, buffer);
-        kdWarning() << buffer << endl;
-        longjmp(myerr->setjmp_buffer, 1);
+        kdDebug() << "Use Exif/Iptc preview extraction. Size of image: " 
+                  << image.width() << "x" << image.height() << endl;
+        return true;
     }
+        
+    // Else, use the right way depending of image file extension.
+    QString ext = fileInfo.extension(false).upper();
+    QString rawFilesExt(raw_file_extentions);
+
+    if (!ext.isEmpty())
+    {
+        if (ext == QString("JPEG") || ext == QString("JPG") || ext == QString("JPE"))
+            return (loadJPEG(image, path));
+        else if (ext == QString("PNG"))
+            return (loadDImg(image, path));
+        else if (ext == QString("TIFF") || ext == QString("TIF"))
+            return (loadDImg(image, path));
+        else if (rawFilesExt.upper().contains(ext))
+            return (KDcrawIface::KDcraw::loadDcrawPreview(image, path));
+    }
+
+    return false;
 }
 
 bool kio_digikamthumbnailProtocol::loadJPEG(QImage& image, const QString& path)
 {
-    QString format = QImageIO::imageFormat(path);
-    if (format !="JPEG") return false;
-
-    FILE* inputFile=fopen(QFile::encodeName(path), "rb");
-    if(!inputFile)
-        return false;
-
-    struct jpeg_decompress_struct cinfo;
-    struct myjpeg_error_mgr       jerr;
-
-    // JPEG error handling - thanks to Marcus Meissner
-    cinfo.err             = jpeg_std_error(&jerr);
-    cinfo.err->error_exit = myjpeg_error_exit;
-
-    if (setjmp(jerr.setjmp_buffer)) 
-    {
-        jpeg_destroy_decompress(&cinfo);
-        fclose(inputFile);
-        return false;
-    }
-
-    jpeg_create_decompress(&cinfo);
-    jpeg_stdio_src(&cinfo, inputFile);
-    jpeg_read_header(&cinfo, true);
-
-    int imgSize = QMAX(cinfo.image_width, cinfo.image_height);
-
-    int scale=1;
-    while(cachedSize_*scale*2<=imgSize) 
-    {
-        scale*=2;
-    }
-    if(scale>8) scale=8;
-
-    cinfo.scale_num=1;
-    cinfo.scale_denom=scale;
-
-    switch (cinfo.jpeg_color_space)
-    {
-        case JCS_UNKNOWN:
-            break;
-        case JCS_GRAYSCALE:
-        case JCS_RGB:
-        case JCS_YCbCr:
-            cinfo.out_color_space = JCS_RGB;
-            break;
-        case JCS_CMYK:
-        case JCS_YCCK:
-            cinfo.out_color_space = JCS_CMYK;
-            break;
-    }
-
-    jpeg_start_decompress(&cinfo);
-
-    QImage img;
-
-    // We only take RGB with 1 or 3 components, or CMYK with 4 components
-    if (!(
-           (cinfo.out_color_space == JCS_RGB  && (cinfo.output_components == 3 || cinfo.output_components == 1))
-        || (cinfo.out_color_space == JCS_CMYK &&  cinfo.output_components == 4)
-        ))
-    {
-        jpeg_destroy_decompress(&cinfo);
-        fclose(inputFile);
-        return false;
-    }
-
-    switch(cinfo.output_components) 
-    {
-        case 3:
-        case 4:
-            img.create( cinfo.output_width, cinfo.output_height, 32 );
-            break;
-        case 1: // B&W image
-            img.create( cinfo.output_width, cinfo.output_height, 8, 256 );
-            for (int i = 0 ; i < 256 ; i++)
-                img.setColor(i, qRgb(i, i, i));
-            break;
-    }
-
-    uchar** lines = img.jumpTable();
-    while (cinfo.output_scanline < cinfo.output_height)
-        jpeg_read_scanlines(&cinfo, lines + cinfo.output_scanline, cinfo.output_height);
-        
-    jpeg_finish_decompress(&cinfo);
-
-    // Expand 24->32 bpp
-    if ( cinfo.output_components == 3 )
-    {
-        for (uint j=0; j<cinfo.output_height; j++) 
-        {
-            uchar *in = img.scanLine(j) + cinfo.output_width*3;
-            QRgb *out = (QRgb*)( img.scanLine(j) );
-
-            for (uint i=cinfo.output_width; i--; ) 
-            {
-                in -= 3;
-                out[i] = qRgb(in[0], in[1], in[2]);
-            }
-        }
-    }
-    else if ( cinfo.output_components == 4 )
-    {
-        // CMYK conversion
-        for (uint j=0; j<cinfo.output_height; j++)
-        {
-            uchar *in = img.scanLine(j) + cinfo.output_width*4;
-            QRgb *out = (QRgb*)( img.scanLine(j) );
-
-            for (uint i=cinfo.output_width; i--; )
-            {
-                in -= 4;
-                int k = in[3];
-                out[i] = qRgb(k * in[0] / 255, k * in[1] / 255, k * in[2] / 255);
-            }
-        }
-    }
-
-    int newMax = QMAX(cinfo.output_width, cinfo.output_height);
-    int newx = cachedSize_*cinfo.output_width / newMax;
-    int newy = cachedSize_*cinfo.output_height / newMax;
-
-    jpeg_destroy_decompress(&cinfo);
-    fclose(inputFile);
-
-    image = img.smoothScale(newx,newy);
-
-    return true;
+    return Digikam::loadJPEGScaled(image, path, cachedSize_);
 }
 
 void kio_digikamthumbnailProtocol::exifRotate(const QString& filePath, QImage& thumb)
 {
-    // Rotate thumbnail based on EXIF rotate tag
+    // Rotate thumbnail based on metadata orientation information
+
+    DMetadata metadata(filePath);
+    DMetadata::ImageOrientation orientation = metadata.getImageOrientation();
+
+    if (orientation == DMetadata::ORIENTATION_NORMAL ||
+        orientation == DMetadata::ORIENTATION_UNSPECIFIED)
+        return;
+
     QWMatrix matrix;
 
-    KExifData ke;
-    ke.readFromFile(filePath);
-    KExifData::ImageOrientation orientation = ke.getImageOrientation();
-    kdDebug() << "Image Orientation: " << (KExifData::ImageOrientation)orientation << endl;
-
-    bool doXform = (orientation != KExifData::NORMAL &&
-                    orientation != KExifData::UNSPECIFIED);
-
-    switch (orientation) 
+    switch (orientation)
     {
-       case KExifData::NORMAL:
-       case KExifData::UNSPECIFIED:
-          break;
+        case DMetadata::ORIENTATION_NORMAL:
+        case DMetadata::ORIENTATION_UNSPECIFIED:
+            break;
 
-       case KExifData::HFLIP:
-          matrix.scale(-1, 1);
-          break;
+        case DMetadata::ORIENTATION_HFLIP:
+            matrix.scale(-1, 1);
+            break;
 
-       case KExifData::ROT_180:
-          matrix.rotate(180);
-          break;
+        case DMetadata::ORIENTATION_ROT_180:
+            matrix.rotate(180);
+            break;
 
-       case KExifData::VFLIP:
-          matrix.scale(1,-1);
-          break;
+        case DMetadata::ORIENTATION_VFLIP:
+            matrix.scale(1, -1);
+            break;
 
-       case KExifData::ROT_90_HFLIP:
-          matrix.scale(-1, 1);
-          matrix.rotate(90);
-          break;
+        case DMetadata::ORIENTATION_ROT_90_HFLIP:
+            matrix.scale(-1, 1);
+            matrix.rotate(90);
+            break;
 
-       case KExifData::ROT_90:
-          matrix.rotate(90);
-          break;
+        case DMetadata::ORIENTATION_ROT_90:
+            matrix.rotate(90);
+            break;
 
-       case KExifData::ROT_90_VFLIP:
-          matrix.scale(1, -1);
-          matrix.rotate(90);
-          break;
+        case DMetadata::ORIENTATION_ROT_90_VFLIP:
+            matrix.scale(1, -1);
+            matrix.rotate(90);
+            break;
 
-       case KExifData::ROT_270:
-          matrix.rotate(270);
-          break;
+        case DMetadata::ORIENTATION_ROT_270:
+            matrix.rotate(270);
+            break;
     }
 
-    //transform accordingly
-    if ( doXform )
-       thumb = thumb.xForm( matrix );
+    // transform accordingly
+    thumb = thumb.xForm( matrix );
 }
 
 QImage kio_digikamthumbnailProtocol::loadPNG(const QString& path)
@@ -552,7 +418,6 @@ QImage kio_digikamthumbnailProtocol::loadPNG(const QString& path)
     if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY)
         has_grey = 1;
 
-
     unsigned char **lines;
     int             i;
 
@@ -619,148 +484,39 @@ QImage kio_digikamthumbnailProtocol::loadPNG(const QString& path)
     return qimage;
 }
 
-// -- Load using Imlib2 API ---------------------------------------------------------------
+// -- Load using DImg ---------------------------------------------------------------------
 
-bool kio_digikamthumbnailProtocol::loadImlib2(QImage& image, const QString& path)
+bool kio_digikamthumbnailProtocol::loadDImg(QImage& image, const QString& path)
 {
-    Imlib_Image imlib2_im = imlib_load_image_immediately_without_cache(QFile::encodeName(path));
+    Digikam::DImg dimg_im;
 
-    if (imlib2_im == NULL) 
+    // to disable raw loader - does not work from ioslave
+    dimg_im.setAttribute("noeventloop", true);
+
+    if (!dimg_im.load(path))
     {
         return false;
     }
 
-    imlib_context_set_image(imlib2_im);
-
-    org_width_  = imlib_image_get_width();
-    org_height_ = imlib_image_get_height();
+    image = dimg_im.copyQImage();
+    org_width_  = image.width();
+    org_height_ = image.height();
 
     if ( QMAX(org_width_, org_height_) != cachedSize_ )
     {
-        imlib2_im = imlib_create_cropped_scaled_image(0, 0,
-                                                      org_width_, org_height_,
-                                                      cachedSize_, cachedSize_);
+        QSize sz(dimg_im.width(), dimg_im.height());
+        sz.scale(cachedSize_, cachedSize_, QSize::ScaleMin);
+        image.scale(sz.width(), sz.height());
     }
 
-    new_width_  = imlib_image_get_width();
-    new_height_ = imlib_image_get_height();
+    new_width_  = image.width();
+    new_height_ = image.height();
 
-    image.create( new_width_, new_height_, 32 );
     image.setAlphaBuffer(true) ;
 
-    DATA32 *data = imlib_image_get_data();
-    if (!data)
-        return false;
-
-    memcpy(image.bits(), data, image.numBytes());
-
-    imlib_free_image();
     return true;
 }
 
-// -- Load using Dcraw ---------------------------------------------------------------------
-
-bool kio_digikamthumbnailProtocol::loadDCRAW(QImage& image, const QString& path)
-{
-    FILE       *f;
-    QByteArray  imgData;
-    const int   MAX_IPC_SIZE = (1024*32);
-    char        buffer[MAX_IPC_SIZE];
-    QFile       file;
-    Q_LONG      len;
-    QCString    command;
-    
-    QFileInfo fileInfo(path);
-    QString rawFilesExt(raw_file_extentions);
-
-    if (!fileInfo.exists() || !rawFilesExt.upper().contains( fileInfo.extension().upper() ))
-        return false;
-
-    // Try to extract embedded thumbnail using dcraw with options:
-    // -c : write to stdout
-    // -e : Extract the camera-generated thumbnail, not the raw image (JPEG or a PPM file).
-    // Note : this code require at least dcraw version 8.x
-
-    command  = "dcraw -c -e ";
-    command += QFile::encodeName( KProcess::quote( path ) );
-    kdDebug() << "Running dcraw command " << command << endl;
-
-    f = popen( command.data(), "r" );
-
-    if ( !f )
-        return false;
-
-    file.open( IO_ReadOnly,  f );
-
-    while ((len = file.readBlock(buffer, MAX_IPC_SIZE)) != 0)
-    {
-        if ( len == -1 )
-        {
-            file.close();
-            return false;
-        }
-        else
-        {
-            int oldSize = imgData.size();
-            imgData.resize( imgData.size() + len );
-            memcpy(imgData.data()+oldSize, buffer, len);
-        }
-    }
-
-    file.close();
-    pclose( f );
-
-    if ( !imgData.isEmpty() )
-    {
-        if (image.loadFromData( imgData ))
-            return true;
-    }
-    
-    // In second, try to use simple RAW extraction method
-    // -c : write to stdout
-    // -h : Half-size color image (3x faster than -q)
-    // -2 : 8bit ppm output
-    // -a : Use automatic white balance
-    // -w : Use camera white balance, if possible
-
-    command  = "dcraw -c -h -2 -w -a ";
-    command += QFile::encodeName( KProcess::quote( path ) );
-    kdDebug() << "Running dcraw command " << command << endl;
-
-    f = popen( command.data(), "r" );
-
-    if ( !f )
-        return false;
-
-    file.open( IO_ReadOnly,  f );
-
-    while ((len = file.readBlock(buffer, MAX_IPC_SIZE)) != 0)
-    {
-        if ( len == -1 )
-        {
-            file.close();
-            return false;
-        }
-        else
-        {
-            int oldSize = imgData.size();
-            imgData.resize( imgData.size() + len );
-            memcpy(imgData.data()+oldSize, buffer, len);
-        }
-    }
-
-    file.close();
-    pclose( f );
-
-    if ( !imgData.isEmpty() )
-    {
-        if (image.loadFromData( imgData ))
-            return true;
-    }
-    
-    return false;
-}
-
 // -- Load using KDE API ---------------------------------------------------------------------
 
 bool kio_digikamthumbnailProtocol::loadKDEThumbCreator(QImage& image, const QString& path)
@@ -770,7 +526,7 @@ bool kio_digikamthumbnailProtocol::loadKDEThumbCreator(QImage& image, const QStr
     // code fixed, we will have to create a qapp instance.
     if (!app_)
         app_ = new QApplication(argc_, argv_);
-    
+
     QString mimeType = KMimeType::findByURL(path)->name();
     if (mimeType.isEmpty())
     {
@@ -779,9 +535,9 @@ bool kio_digikamthumbnailProtocol::loadKDEThumbCreator(QImage& image, const QStr
     }
 
     QString mimeTypeAlt = mimeType.replace(QRegExp("/.*"), "/*");
-    
+
     QString plugin;
-    
+
     KTrader::OfferList plugins = KTrader::self()->query("ThumbCreator");
     for (KTrader::OfferList::ConstIterator it = plugins.begin(); it != plugins.end(); ++it)
     {
@@ -794,7 +550,7 @@ bool kio_digikamthumbnailProtocol::loadKDEThumbCreator(QImage& image, const QStr
                 break;
             }
         }
-        
+
         if (!plugin.isEmpty())
             break;
     }
@@ -845,8 +601,7 @@ void kio_digikamthumbnailProtocol::createThumbnailDirs()
     KStandardDirs::makeDir(bigThumbPath_, 0700);
 }
 
-/////////////////////////////////////////////////////////////////////////////////////////
-// KIO slave registration
+// -- KIO slave registration ---------------------------------------------------------------------
 
 extern "C"
 {
@@ -856,7 +611,8 @@ extern "C"
         KInstance instance( "kio_digikamthumbnail" );
         ( void ) KGlobal::locale();
 
-        if (argc != 4) {
+        if (argc != 4) 
+        {
             kdDebug() << "Usage: kio_digikamthumbnail  protocol domain-socket1 domain-socket2"
                       << endl;
             exit(-1);
