@@ -1,12 +1,14 @@
 /* ============================================================
- * Authors: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- *          Caulier Gilles <caulier dot gilles at gmail dot com>
- *          Marcel Wiesweg <marcel.wiesweg@gmx.de>
- * Date   : 2003-03-09
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ *
+ * Date        : 2003-03-09
  * Description : Comments, Tags, and Rating properties editor
  *
- * Copyright 2003-2005 by Renchi Raju & Gilles Caulier
- * Copyright 2006-2007 by Gilles Caulier & Marcel Wiesweg
+ * Copyright (C) 2003-2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
+ * Copyright (C) 2003-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2007 by Marcel Wiesweg <marcel.wiesweg@gmx.de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -68,6 +70,7 @@
 #include "imageattributeswatch.h"
 #include "metadatahub.h"
 #include "statusprogressbar.h"
+#include "searchtextbar.h"
 #include "imagedescedittab.h"
 #include "imagedescedittab.moc"
 
@@ -84,9 +87,8 @@ public:
         modified                   = false;
         ignoreImageAttributesWatch = false;
         recentTagsBtn              = 0;
-        tagsSearchClearBtn         = 0;
         commentsEdit               = 0;
-        tagsSearchEdit             = 0;
+        tagsSearchBar              = 0;
         dateTimeEdit               = 0;
         tagsView                   = 0;
         ratingWidget               = 0;
@@ -101,7 +103,6 @@ public:
     bool                           ignoreImageAttributesWatch;
 
     QToolButton                   *recentTagsBtn;
-    QToolButton                   *tagsSearchClearBtn;
     QToolButton                   *assignedTagsBtn;
     QToolButton                   *revertBtn;
 
@@ -114,10 +115,10 @@ public:
 
     KTextEdit                     *commentsEdit;
 
-    KLineEdit                     *tagsSearchEdit;
-
     KDateTimeEdit                 *dateTimeEdit;
 
+    SearchTextBar                 *tagsSearchBar;
+
     QPtrList<ImageInfo>            currInfos;
 
     TAlbumListView                *tagsView;
@@ -145,7 +146,7 @@ ImageDescEditTab::ImageDescEditTab(QWidget *parent, bool navBar)
     // Comments/Date/Rating view -----------------------------------
     
     QVBox *commentsBox = new QVBox(settingsArea);
-    new QLabel(i18n("Comments:"), commentsBox);
+    new QLabel(i18n("Caption:"), commentsBox);
     d->commentsEdit = new KTextEdit(commentsBox);
     d->commentsEdit->setTextFormat(QTextEdit::PlainText);
     d->commentsEdit->setCheckSpellingEnabled(true);
@@ -160,17 +161,10 @@ ImageDescEditTab::ImageDescEditTab(QWidget *parent, bool navBar)
 
     // Tags view ---------------------------------------------------
 
-    QHBox *tagsSearch = new QHBox(settingsArea);
+    QHBox *tagsSearch  = new QHBox(settingsArea);
     tagsSearch->setSpacing(KDialog::spacingHint());
 
-    d->tagsSearchClearBtn = new QToolButton(tagsSearch);
-    d->tagsSearchClearBtn->setAutoRaise(true);
-    d->tagsSearchClearBtn->setIconSet(kapp->iconLoader()->loadIcon("locationbar_erase",
-                                      KIcon::Toolbar, KIcon::SizeSmall));
-
-    new QLabel(i18n("Search:"), tagsSearch);
-    d->tagsSearchEdit = new KLineEdit(tagsSearch);
-    d->tagsSearchEdit->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
+    d->tagsSearchBar   = new SearchTextBar(tagsSearch);
 
     d->assignedTagsBtn = new QToolButton(tagsSearch);
     QToolTip::add(d->assignedTagsBtn, i18n("Already assigned tags"));
@@ -204,11 +198,11 @@ ImageDescEditTab::ImageDescEditTab(QWidget *parent, bool navBar)
     d->applyBtn = new QPushButton(i18n("Apply"), buttonsBox);
     d->applyBtn->setIconSet(SmallIcon("button_ok"));
     d->applyBtn->setEnabled(false);
-    QToolTip::add(d->applyBtn, i18n("Apply all changes to pictures"));
+    QToolTip::add(d->applyBtn, i18n("Apply all changes to images"));
     buttonsBox->setStretchFactor(d->applyBtn, 10);
 
     d->moreButton = new QPushButton(i18n("More"), buttonsBox);
-    d->moreMenu = new QPopupMenu(this);
+    d->moreMenu   = new QPopupMenu(this);
     d->moreButton->setPopup(d->moreMenu);
 
     // --------------------------------------------------
@@ -247,11 +241,11 @@ ImageDescEditTab::ImageDescEditTab(QWidget *parent, bool navBar)
     connect(d->tagsView, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint &, int)),
             this, SLOT(slotRightButtonClicked(QListViewItem*, const QPoint&, int)));
             
-    connect(d->tagsSearchClearBtn, SIGNAL(clicked()),
-            d->tagsSearchEdit, SLOT(clear()));
-            
-    connect(d->tagsSearchEdit, SIGNAL(textChanged(const QString&)),
-            this, SLOT(slotTagsSearchChanged()));
+    connect(d->tagsSearchBar, SIGNAL(signalTextChanged(const QString&)),
+            this, SLOT(slotTagsSearchChanged(const QString&)));
+
+    connect(this, SIGNAL(signalTagFilterMatch(bool)),
+            d->tagsSearchBar, SLOT(slotSearchResult(bool)));
 
     connect(d->assignedTagsBtn, SIGNAL(toggled(bool)),
             this, SLOT(slotAssignedTagsToggled(bool)));
@@ -303,6 +297,9 @@ ImageDescEditTab::ImageDescEditTab(QWidget *parent, bool navBar)
     connect(loader, SIGNAL(signalFailed(Album *)),
             this, SLOT(slotThumbnailLost(Album *)));
 
+    connect(loader, SIGNAL(signalReloadThumbnails()),
+            this, SLOT(slotReloadThumbnails()));
+
     ImageAttributesWatch *watch = ImageAttributesWatch::instance();
 
     connect(watch, SIGNAL(signalImageTagsChanged(Q_LLONG)),
@@ -384,28 +381,28 @@ void ImageDescEditTab::slotChangingItems()
         if (changedFields == 1)
         {
             if (d->hub.commentChanged())
-                text = i18n("<qt><p>You have edited the comment of the picture. ",
-                            "<qt><p>You have edited the comment of %n pictures. ",
+                text = i18n("<qt><p>You have edited the comment of the image. ",
+                            "<qt><p>You have edited the comment of %n images. ",
                             d->currInfos.count());
             else if (d->hub.dateTimeChanged())
-                text = i18n("<qt><p>You have edited the date of the picture. ",
-                            "<qt><p>You have edited the date of %n pictures. ",
+                text = i18n("<qt><p>You have edited the date of the image. ",
+                            "<qt><p>You have edited the date of %n images. ",
                             d->currInfos.count());
             else if (d->hub.ratingChanged())
-                text = i18n("<qt><p>You have edited the rating of the picture. ",
-                            "<qt><p>You have edited the rating of %n pictures. ",
+                text = i18n("<qt><p>You have edited the rating of the image. ",
+                            "<qt><p>You have edited the rating of %n images. ",
                             d->currInfos.count());
             else if (d->hub.tagsChanged())
-                text = i18n("<qt><p>You have edited the tags of the picture. ",
-                            "<qt><p>You have edited the tags of %n pictures. ",
+                text = i18n("<qt><p>You have edited the tags of the image. ",
+                            "<qt><p>You have edited the tags of %n images. ",
                             d->currInfos.count());
 
             text += i18n("Do you want to apply your changes?</p></qt>");
         }
         else
         {
-            text = i18n("<qt><p>You have edited the metadata of the picture: </p><ul>",
-                        "<qt><p>You have edited the metadata of %n pictures: </p><ul>",
+            text = i18n("<qt><p>You have edited the metadata of the image: </p><ul>",
+                        "<qt><p>You have edited the metadata of %n images: </p><ul>",
                         d->currInfos.count());
 
             if (d->hub.commentChanged())
@@ -450,7 +447,7 @@ void ImageDescEditTab::slotApplyAllChanges()
 
     bool progressInfo = (d->currInfos.count() > 1);
     emit signalProgressBarMode(StatusProgressBar::ProgressBarMode,
-                               i18n("Applying changes to pictures. Please wait..."));
+                               i18n("Applying changes to images. Please wait..."));
     MetadataWriteSettings writeSettings = MetadataHub::defaultWriteSettings();
 
     // debugging - use this to indicate reentry from event loop (kapp->processEvents)
@@ -815,7 +812,7 @@ void ImageDescEditTab::slotRightButtonClicked(QListViewItem *item, const QPoint
     if (!album->isRoot())
     {
         selectTagsMenu.insertSeparator(-1);
-        selectTagsMenu.insertItem(i18n("Childs"),     17);
+        selectTagsMenu.insertItem(i18n("Children"),     17);
         selectTagsMenu.insertItem(i18n("Parents"),    19);
     }
     popmenu.insertItem(i18n("Select"), &selectTagsMenu);
@@ -825,7 +822,7 @@ void ImageDescEditTab::slotRightButtonClicked(QListViewItem *item, const QPoint
     if (!album->isRoot())
     {
         deselectTagsMenu.insertSeparator(-1);
-        deselectTagsMenu.insertItem(i18n("Childs"),   18);
+        deselectTagsMenu.insertItem(i18n("Children"),   18);
         deselectTagsMenu.insertItem(i18n("Parents"),  20);
     }
     popmenu.insertItem(i18n("Deselect"), &deselectTagsMenu);
@@ -837,11 +834,11 @@ void ImageDescEditTab::slotRightButtonClicked(QListViewItem *item, const QPoint
     toggleAutoMenu.setCheckable(true);
     toggleAutoMenu.insertItem(i18n("None"),    21);
     toggleAutoMenu.insertSeparator(-1);
-    toggleAutoMenu.insertItem(i18n("Childs"),  22);
+    toggleAutoMenu.insertItem(i18n("Children"),  22);
     toggleAutoMenu.insertItem(i18n("Parents"), 23);
     toggleAutoMenu.insertItem(i18n("Both"),    24);
     toggleAutoMenu.setItemChecked(21 + d->toggleAutoTags, true);
-    popmenu.insertItem(i18n("Toogle Auto"), &toggleAutoMenu);
+    popmenu.insertItem(i18n("Toggle Auto"), &toggleAutoMenu);
 
     TagFilterView::ToggleAutoTags oldAutoTags = d->toggleAutoTags;            
 
@@ -954,7 +951,7 @@ void ImageDescEditTab::slotRightButtonClicked(QListViewItem *item, const QPoint
             d->toggleAutoTags = TagFilterView::NoToggleAuto;
             break;
         }
-        case 22:   // Toggle auto Childs tags.
+        case 22:   // Toggle auto Children tags.
         {
             d->toggleAutoTags = TagFilterView::Children;
             break;
@@ -964,7 +961,7 @@ void ImageDescEditTab::slotRightButtonClicked(QListViewItem *item, const QPoint
             d->toggleAutoTags = TagFilterView::Parents;
             break;
         }
-        case 24:   // Toggle auto Childs and Parents tags.
+        case 24:   // Toggle auto Children and Parents tags.
         {
             d->toggleAutoTags = TagFilterView::ChildrenAndParents;
             break;
@@ -1291,7 +1288,7 @@ void ImageDescEditTab::setTagThumbnail(TAlbum *album)
     {
         if (icon.isNull())
         {
-            item->setPixmap(0, loader->getStandardTagIcon(album, 20));
+            item->setPixmap(0, loader->getStandardTagIcon(album, AlbumThumbnailLoader::SmallerSize));
         }
         else
         {
@@ -1326,6 +1323,16 @@ void ImageDescEditTab::slotThumbnailLost(Album *)
     // we already set the standard icon before loading
 }
 
+void ImageDescEditTab::slotReloadThumbnails()
+{
+    AlbumList tList = AlbumManager::instance()->allTAlbums();
+    for (AlbumList::iterator it = tList.begin(); it != tList.end(); ++it)
+    {
+        TAlbum* tag  = (TAlbum*)(*it);
+        setTagThumbnail(tag);
+    }
+}
+
 void ImageDescEditTab::slotImageTagsChanged(Q_LLONG imageId)
 {
     // don't lose modifications
@@ -1423,7 +1430,7 @@ void ImageDescEditTab::updateRecentTags()
                 {
                     if (icon.isNull())
                     {
-                        icon = loader->getStandardTagIcon(album, 20);
+                        icon = loader->getStandardTagIcon(album, AlbumThumbnailLoader::SmallerSize);
                     }
                 }
                 QString text = album->title() + " (" + ((TAlbum*)album->parent())->prettyURL() + ')';
@@ -1453,11 +1460,10 @@ void ImageDescEditTab::slotRecentTagsMenuActivated(int id)
     }
 }
 
-void ImageDescEditTab::slotTagsSearchChanged()
+void ImageDescEditTab::slotTagsSearchChanged(const QString& filter)
 {
     //TODO: this will destroy assigned-tags filtering. Unify in one method.
-    QString search(d->tagsSearchEdit->text());
-    search = search.lower();
+    QString search = filter.lower();
 
     bool atleastOneMatch = false;
 
@@ -1522,7 +1528,6 @@ void ImageDescEditTab::slotTagsSearchChanged()
 
     if (search.isEmpty())
     {
-        d->tagsSearchEdit->unsetPalette();
         TAlbum* root = AlbumManager::instance()->findTAlbum(0);
         TAlbumCheckListItem* rootItem = (TAlbumCheckListItem*)(root->extraData(this));
         if (rootItem)
@@ -1534,13 +1539,9 @@ void ImageDescEditTab::slotTagsSearchChanged()
         TAlbumCheckListItem* rootItem = (TAlbumCheckListItem*)(root->extraData(this));
         if (rootItem)
             rootItem->setText(0, i18n("Found Tags"));
-
-        QPalette pal = d->tagsSearchEdit->palette();
-        pal.setColor(QPalette::Active, QColorGroup::Base,
-                     atleastOneMatch ?  QColor(200,255,200) :
-                     QColor(255,200,200));
-        d->tagsSearchEdit->setPalette(pal);
     }
+
+    emit signalTagFilterMatch(atleastOneMatch);
 }
 
 void ImageDescEditTab::slotAssignedTagsToggled(bool t)
