@@ -1,9 +1,12 @@
 /* ============================================================
- * Authors: Caulier Gilles <caulier dot gilles at gmail dot com>
- * Date   : 2004-11-17
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ *
+ * Date        : 2004-11-17
  * Description : a tab to display colors information of images
  *
- * Copyright 2004-2007 by Gilles Caulier
+ * Copyright (C) 2004-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -33,6 +36,7 @@
 #include <qhbuttongroup.h> 
 #include <qpushbutton.h>
 #include <qtooltip.h>
+#include <qvbox.h>
 
 // KDE includes.
 
@@ -95,9 +99,11 @@ public:
         hGradient            = 0;
         histogramWidget      = 0;
         imageLoaderThread    = 0;
+
+        inLoadingProcess     = false;
     }
 
-    bool                   blinkFlag;
+    bool                   inLoadingProcess;
 
     QComboBox             *channelCB;
     QComboBox             *colorsCB;
@@ -148,7 +154,8 @@ ImagePropertiesColorsTab::ImagePropertiesColorsTab(QWidget* parent, bool navBar)
     // Histogram tab area -----------------------------------------------------
 
     QWidget* histogramPage = new QWidget( d->tab );
-    QGridLayout *topLayout = new QGridLayout(histogramPage, 8, 3, KDialog::marginHint(), KDialog::spacingHint());
+    QGridLayout *topLayout = new QGridLayout(histogramPage, 8, 3, 
+                                 KDialog::spacingHint(), KDialog::spacingHint());
 
     QLabel *label1 = new QLabel(i18n("Channel:"), histogramPage);
     label1->setAlignment ( Qt::AlignRight | Qt::AlignVCenter );
@@ -243,14 +250,15 @@ ImagePropertiesColorsTab::ImagePropertiesColorsTab(QWidget* parent, bool navBar)
 
     // -------------------------------------------------------------
 
-    d->histogramWidget = new HistogramWidget(256, 140, histogramPage);
+    QVBox *histoBox    = new QVBox(histogramPage);
+    d->histogramWidget = new HistogramWidget(256, 140, histoBox);
     QWhatsThis::add( d->histogramWidget, i18n("<p>This is the histogram drawing of the "
                                               "selected image channel"));
-
-    d->hGradient = new ColorGradientWidget( ColorGradientWidget::Horizontal, 10, histogramPage );
-    d->hGradient->setColors( QColor( "black" ), QColor( "white" ) );
-    topLayout->addMultiCellWidget(d->histogramWidget, 3, 3, 0, 3);
-    topLayout->addMultiCellWidget(d->hGradient, 4, 4, 0, 3);
+    QLabel *space = new QLabel(histoBox);
+    space->setFixedHeight(1);
+    d->hGradient = new ColorGradientWidget(ColorGradientWidget::Horizontal, 10, histoBox);
+    d->hGradient->setColors(QColor("black"), QColor("white"));
+    topLayout->addMultiCellWidget(histoBox, 3, 4, 0, 3);
 
     // -------------------------------------------------------------
 
@@ -277,35 +285,35 @@ ImagePropertiesColorsTab::ImagePropertiesColorsTab(QWidget* parent, bool navBar)
                                 "selected histogram part. These values are available for all "
                                 "channels."));
 
-    QLabel *label4 = new QLabel(i18n("Mean:"), gbox);
-    label4->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
-    d->labelMeanValue = new QLabel(gbox);
-    d->labelMeanValue->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
-
     QLabel *label5 = new QLabel(i18n("Pixels:"), gbox);
-    label5->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    label5->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
     d->labelPixelsValue = new QLabel(gbox);
-    d->labelPixelsValue->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
-
-    QLabel *label6 = new QLabel(i18n("Standard deviation:"), gbox);
-    label6->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
-    d->labelStdDevValue = new QLabel(gbox);
-    d->labelStdDevValue->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelPixelsValue->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
     QLabel *label7 = new QLabel(i18n("Count:"), gbox);
-    label7->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    label7->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
     d->labelCountValue = new QLabel(gbox);
-    d->labelCountValue->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelCountValue->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+
+    QLabel *label4 = new QLabel(i18n("Mean:"), gbox);
+    label4->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelMeanValue = new QLabel(gbox);
+    d->labelMeanValue->setAlignment (Qt::AlignRight | Qt::AlignVCenter);
+
+    QLabel *label6 = new QLabel(i18n("Std. deviation:"), gbox);
+    label6->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelStdDevValue = new QLabel(gbox);
+    d->labelStdDevValue->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
     QLabel *label8 = new QLabel(i18n("Median:"), gbox);
-    label8->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    label8->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
     d->labelMedianValue = new QLabel(gbox);
-    d->labelMedianValue->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelMedianValue->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
     QLabel *label9 = new QLabel(i18n("Percentile:"), gbox);
-    label9->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    label9->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
     d->labelPercentileValue = new QLabel(gbox);
-    d->labelPercentileValue->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelPercentileValue->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
     topLayout->addMultiCellWidget(gbox, 6, 6, 0, 3);
 
@@ -313,15 +321,16 @@ ImagePropertiesColorsTab::ImagePropertiesColorsTab(QWidget* parent, bool navBar)
 
     QGroupBox *gbox2 = new QGroupBox(2, Qt::Horizontal, histogramPage);
     gbox2->setFrameStyle( QFrame::NoFrame );
+    gbox2->setInsideMargin(0);
 
     QLabel *label11     = new QLabel(i18n("Color depth:"), gbox2);
-    label11->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
-    d->labelColorDepth   = new QLabel(gbox2);
-    d->labelColorDepth->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    label11->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelColorDepth  = new QLabel(gbox2);
+    d->labelColorDepth->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     QLabel *label12     = new QLabel(i18n("Alpha Channel:"), gbox2);
-    label12->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    label12->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
     d->labelAlphaChannel = new QLabel(gbox2);
-    d->labelAlphaChannel->setAlignment ( Qt::AlignLeft | Qt::AlignVCenter);
+    d->labelAlphaChannel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
     topLayout->addMultiCellWidget(gbox2, 7, 7, 0, 3);
 
@@ -417,7 +426,7 @@ void ImagePropertiesColorsTab::setData(const KURL& url, const QRect &selectionAr
     // More importantly, while the loading thread can handle this pretty well,
     // this will completely mess up the timing of progress info in the histogram widget.
     // So filter here, before the stopHistogramComputation!
-    if (!img && url.path() == d->currentFilePath)
+    if (!img && url.path() == d->currentFilePath && d->inLoadingProcess)
         return;
 
     // This is necessary to stop computation because d->image.bits() is currently used by
@@ -508,7 +517,7 @@ void ImagePropertiesColorsTab::loadImageFromUrl(const KURL& url)
     {
         // use raw settings optimized for speed
 
-        RawDecodingSettings rawDecodingSettings = RawDecodingSettings();
+        KDcrawIface::RawDecodingSettings rawDecodingSettings = KDcrawIface::RawDecodingSettings();
         rawDecodingSettings.optimizeTimeLoading();
         desc = LoadingDescription(desc.filePath, rawDecodingSettings);
     }
@@ -518,6 +527,7 @@ void ImagePropertiesColorsTab::loadImageFromUrl(const KURL& url)
 
     d->currentFilePath = desc.filePath;
     d->currentLoadingDescription = desc;
+    d->inLoadingProcess = true;
 
     d->imageLoaderThread->load(d->currentLoadingDescription,
                                SharedLoadSaveThread::AccessModeRead,
@@ -550,6 +560,7 @@ void ImagePropertiesColorsTab::slotLoadImageFromUrlComplete(const LoadingDescrip
         d->iccProfileWidget->setLoadingFailed();
         slotHistogramComputationFailed();
     }
+    d->inLoadingProcess = false;
 }
 
 void ImagePropertiesColorsTab::slotMoreCompleteLoadingAvailable(const LoadingDescription &oldLoadingDescription,
@@ -562,6 +573,7 @@ void ImagePropertiesColorsTab::slotMoreCompleteLoadingAvailable(const LoadingDes
         // Even the time-optimized raw loading takes significant time, and we must avoid two dcraw instances running
         // at a time.
         d->currentLoadingDescription = newLoadingDescription;
+        d->inLoadingProcess = true;
         d->imageLoaderThread->load(newLoadingDescription,
                                    SharedLoadSaveThread::AccessModeRead,
                                    SharedLoadSaveThread::LoadingPolicyFirstRemovePrevious);
