@@ -1,11 +1,13 @@
 /* ============================================================
- * Authors: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- *          Gilles Caulier <caulier dot gilles at gmail dot com>
- * Date   : 2003-02-10
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ *
+ * Date        : 2003-02-10
  * Description : Camera type selection dialog
  * 
- * Copyright 2003-2005 by Renchi Raju
- * Copyright 2006 by Gilles Caulier
+ * Copyright (C) 2003-2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
+ * Copyright (C) 2006-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -29,8 +31,6 @@
 #include <qradiobutton.h>
 #include <qvbuttongroup.h>
 #include <qlayout.h>
-#include <qstring.h>
-#include <qstringlist.h>
 #include <qwhatsthis.h>
 
 // KDE includes.
@@ -47,8 +47,9 @@
 
 // Local includes.
 
+#include "gpcamera.h"
 #include "cameraselection.h"
-#include "gpiface.h"
+#include "cameraselection.moc"
 
 namespace Digikam
 {
@@ -95,8 +96,8 @@ CameraSelection::CameraSelection( QWidget* parent )
                : KDialogBase(Plain, i18n("Camera Configuration"),
                              Help|Ok|Cancel, Ok, parent, 0, true, true)
 {
-    kapp->setOverrideCursor( KCursor::waitCursor() );
     d = new CameraSelectionPriv;
+    kapp->setOverrideCursor( KCursor::waitCursor() );
     setHelp("cameraselection.anchor", "digikam");
     d->UMSCameraNameActual = QString("Directory Browse");   // Don't be i18n!
     d->UMSCameraNameShown  = i18n("Mounted Camera");
@@ -184,7 +185,7 @@ CameraSelection::CameraSelection( QWidget* parent )
 
     KActiveLabel* link2 = new KActiveLabel(box2);
     link2->setText(i18n("<p>To set a <b>Generic PTP USB Device</b><br>"
-                        "(which use Picture Transfer Protocol), please<br>"
+                        "(which use Image Transfer Protocol), please<br>"
                         "use <a href=\"ptpcamera\">%1</a> from camera list.</p>")
                         .arg(d->PTPCameraNameShown));
                            
@@ -303,7 +304,7 @@ void CameraSelection::getCameraList()
     QStringList clist;
     QString cname;
     
-    GPIface::getSupportedCameras(count, clist);
+    GPCamera::getSupportedCameras(count, clist);
     
     for (int i = 0 ; i < count ; i++) 
     {
@@ -319,7 +320,7 @@ void CameraSelection::getSerialPortList()
 {
     QStringList plist;
 
-    GPIface::getSupportedPorts(plist);
+    GPCamera::getSupportedPorts(plist);
 
     d->serialPortList.clear();
     
@@ -368,7 +369,7 @@ void CameraSelection::slotSelectionChanged(QListViewItem *item)
     d->titleEdit->setText(model);
     
     QStringList plist;
-    GPIface::getCameraSupportedPorts(model, plist);
+    GPCamera::getCameraSupportedPorts(model, plist);
 
     if (plist.contains("serial")) 
     {
@@ -452,4 +453,3 @@ void CameraSelection::slotOkClicked()
 
 }  // namespace Digikam
 
-#include "cameraselection.moc"
