@@ -1,9 +1,12 @@
 /* ============================================================
- * Authors: Gilles Caulier <caulier dot gilles at gmail dot com>
- * Date   : 2004-08-03
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ * 
+ * Date        : 2004-08-03
  * Description : setup Image Editor tab.
  *
- * Copyright 2004-2007 by Gilles Caulier
+ * Copyright (C) 2004-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -25,10 +28,6 @@
 
 #include <qwidget.h>
 
-class QCheckBox;
-
-class KColorButton;
-
 namespace Digikam
 {
 
@@ -56,7 +55,6 @@ private:
 private:
 
     SetupEditorPriv* d;
-
 };
 
 }  // namespace Digikam
