@@ -1,10 +1,13 @@
 /* ============================================================
- * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
- *         Gilles Caulier <caulier dot gilles at free.fr>
- * Date  : 2003-02-01
- * Description :
  *
- * Copyright 2003-2004 by Renchi Raju
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ *
+ * Date        : 2003-02-01
+ * Description : general configuration setup tab
+ *
+ * Copyright (C) 2003-2004 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
+ * Copyright (C) 2005-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -19,7 +22,6 @@
  *
  * ============================================================ */
 
-
 #ifndef SETUPGENERAL_H
 #define SETUPGENERAL_H
 
@@ -27,21 +29,20 @@
 
 #include <qwidget.h>
 
-class QComboBox;
-class QRadioButton;
-class QCheckBox;
-class QLineEdit;
-
 class KDialogBase;
 
+namespace Digikam
+{
+
+class SetupGeneralPriv;
+
 class SetupGeneral : public QWidget
 {
     Q_OBJECT
 
 public:
-    enum ThumbnailSizes { /*TinyThumb, */SmallThumb = 0, MediumThumb, LargeThumb, HugeThumb };
 
-    SetupGeneral(QWidget* parent = 0, KDialogBase* dialog = 0 );
+    SetupGeneral(QWidget* parent = 0, KDialogBase* dialog = 0);
     ~SetupGeneral();
 
     void applySettings();
@@ -50,28 +51,16 @@ private:
 
     void readSettings();
 
-    QLineEdit    *albumPathEdit;
-
-    QComboBox    *thumbnailSize_;
-
-    QCheckBox    *recurseTagsBox_;
-    QCheckBox    *showToolTipsBox_;
-
-    QCheckBox    *iconShowNameBox_;
-    QCheckBox    *iconShowSizeBox_;
-    QCheckBox    *iconShowDateBox_;
-    QCheckBox    *iconShowResolutionBox_;
-    QCheckBox    *iconShowCommentsBox_;
-    QCheckBox    *iconShowTagsBox_;
-    QCheckBox    *iconShowFileCommentsBox_;
-
-    KDialogBase  *mainDialog_;
-
 private slots:
 
-    void slotChangeAlbumPath();
-    void slotPathEdited(const QString& newPath);
+    void slotChangeAlbumPath(const QString &);
+    void slotPathEdited(const QString&);
 
+private:
+
+    SetupGeneralPriv* d;
 };
 
+}  // namespace Digikam
+
 #endif // SETUPGENERAL_H
