@@ -1,9 +1,12 @@
 /* ============================================================
- * Authors: Gilles Caulier <caulier dot gilles at gmail dot com>
- * Date   : 2005-04-21
+ *
+ * This file is a part of digiKam project
+ * http://www.digikam.org
+ * 
+ * Date        : 2005-04-21
  * Description : slide show tool using preview of pictures.
  * 
- * Copyright 2005-2007 by Gilles Caulier
+ * Copyright (C) 2005-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -41,6 +44,7 @@
 // Local includes.
 
 #include "ddebug.h"
+#include "dimg.h"
 #include "toolbar.h"
 #include "previewloadthread.h"
 #include "slideshow.h"
@@ -78,7 +82,7 @@ public:
 
     QPixmap            pixmap;
 
-    QImage             preview;
+    DImg               preview;
 
     KURL               currentImage;
 
@@ -146,8 +150,8 @@ SlideShow::SlideShow(const SlideShowSettings& settings)
     d->timer                = new QTimer();
     d->mouseMoveTimer       = new QTimer();
 
-    connect(d->previewThread, SIGNAL(signalPreviewLoaded(const LoadingDescription &, const QImage &)),
-            this, SLOT(slotGotImagePreview(const LoadingDescription &, const QImage&)));
+    connect(d->previewThread, SIGNAL(signalImageLoaded(const LoadingDescription &, const DImg &)),
+            this, SLOT(slotGotImagePreview(const LoadingDescription &, const DImg&)));
 
     connect(d->mouseMoveTimer, SIGNAL(timeout()),
             this, SLOT(slotMouseMoveTimeOut()));
@@ -218,7 +222,7 @@ void SlideShow::loadNextImage()
     else
     {
         d->currentImage = KURL();
-        d->preview = QImage();
+        d->preview = DImg();
         updatePixmap();
         update();
     }
@@ -253,14 +257,14 @@ void SlideShow::loadPrevImage()
     else
     {
         d->currentImage = KURL();
-        d->preview = QImage();
+        d->preview = DImg();
         updatePixmap();
         update();
     }
 
 }
 
-void SlideShow::slotGotImagePreview(const LoadingDescription&, const QImage& preview)
+void SlideShow::slotGotImagePreview(const LoadingDescription&, const DImg& preview)
 {
     d->preview = preview;
 
@@ -269,7 +273,8 @@ void SlideShow::slotGotImagePreview(const LoadingDescription&, const QImage& pre
 
     if (!d->endOfShow)
     {
-        d->timer->start(d->settings.delay, true);
+        if (!d->pause)
+            d->timer->start(d->settings.delay, true);
         preloadNextImage();
     }
 }
@@ -306,7 +311,7 @@ void SlideShow::updatePixmap()
         {
             // Preview extraction is complete... Draw the image.
 
-            QPixmap pix(d->preview.smoothScale(size(), QImage::ScaleMin));
+            QPixmap pix(d->preview.smoothScale(width(), height(), QSize::ScaleMin).convertToPixmap());
             p.drawPixmap((width()-pix.width())/2,
                          (height()-pix.height())/2, pix,
                          0, 0, pix.width(), pix.height());
@@ -425,7 +430,7 @@ void SlideShow::updatePixmap()
             p.setPen(Qt::white);
             p.drawText(0, 0, d->pixmap.width(), d->pixmap.height(),
                        Qt::AlignCenter|Qt::WordBreak, 
-                       i18n("Cannot display picture\n\"%1\"")
+                       i18n("Cannot display image\n\"%1\"")
                        .arg(d->currentImage.fileName()));
         }
     }
@@ -445,15 +450,12 @@ void SlideShow::updatePixmap()
         p.drawPixmap(50, 100, logo);
         p.drawText(60 + logo.width(), 100 + logo.height()/3,   i18n("SlideShow Completed."));
         p.drawText(60 + logo.width(), 100 + 2*logo.height()/3, i18n("Click To Exit..."));
-        p.end();
 
         d->endOfShow = true;
         d->toolBar->setEnabledPlay(false);
         d->toolBar->setEnabledNext(false);
         d->toolBar->setEnabledPrev(false);
     }
-
-    p.end();
 }
 
 void SlideShow::printInfoText(QPainter &p, int &offset, const QString& str)
